/*     */ package net.rubyeye.xmemcached.command.binary;
/*     */ 
/*     */ import com.google.code.yanf4j.buffer.IoBuffer;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.concurrent.CountDownLatch;
/*     */ import net.rubyeye.xmemcached.command.Command;
/*     */ import net.rubyeye.xmemcached.command.CommandType;
/*     */ import net.rubyeye.xmemcached.exception.MemcachedDecodeException;
/*     */ import net.rubyeye.xmemcached.exception.MemcachedServerException;
/*     */ import net.rubyeye.xmemcached.exception.UnknownCommandException;
/*     */ import net.rubyeye.xmemcached.impl.MemcachedTCPSession;
/*     */ import net.rubyeye.xmemcached.transcoders.CachedData;
/*     */ import net.rubyeye.xmemcached.transcoders.Transcoder;
/*     */ import net.rubyeye.xmemcached.utils.ByteUtils;
/*     */ import net.rubyeye.xmemcached.utils.OpaqueGenerater;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class BaseBinaryCommand
/*     */   extends Command
/*     */ {
/*     */   static final short DEFAULT_VBUCKET_ID = 0;
/*     */   protected int expTime;
/*     */   protected long cas;
/*     */   protected Object value;
/*     */   protected OpCode opCode;
/*  54 */   protected BinaryDecodeStatus decodeStatus = BinaryDecodeStatus.NONE;
/*     */   protected int responseKeyLength;
/*     */   protected int responseExtrasLength;
/*     */   protected int responseTotalBodyLength;
/*     */   protected ResponseStatus responseStatus;
/*  59 */   protected int opaque; protected short vbucketId = 0;
/*     */   
/*     */   static final byte EXTRAS_LENGTH = 8;
/*     */   
/*     */   public BaseBinaryCommand(String key, byte[] keyBytes, CommandType cmdType, CountDownLatch latch, int exp, long cas, Object value, boolean noreply, Transcoder transcoder)
/*     */   {
/*  65 */     super(key, keyBytes, cmdType, latch);
/*  66 */     this.expTime = exp;
/*  67 */     this.cas = cas;
/*  68 */     this.value = value;
/*  69 */     this.noreply = noreply;
/*  70 */     this.transcoder = transcoder;
/*     */   }
/*     */   
/*     */   public final int getExpTime() {
/*  74 */     return this.expTime;
/*     */   }
/*     */   
/*     */   public final void setExpTime(int exp) {
/*  78 */     this.expTime = exp;
/*     */   }
/*     */   
/*     */   public final long getCas() {
/*  82 */     return this.cas;
/*     */   }
/*     */   
/*     */   public final void setCas(long cas) {
/*  86 */     this.cas = cas;
/*     */   }
/*     */   
/*     */   public final Object getValue() {
/*  90 */     return this.value;
/*     */   }
/*     */   
/*     */   public final void setValue(Object value) {
/*  94 */     this.value = value;
/*     */   }
/*     */   
/*     */ 
/*     */   public final Transcoder getTranscoder()
/*     */   {
/* 100 */     return this.transcoder;
/*     */   }
/*     */   
/*     */ 
/*     */   public final void setTranscoder(Transcoder transcoder)
/*     */   {
/* 106 */     this.transcoder = transcoder;
/*     */   }
/*     */   
/*     */   public boolean decode(MemcachedTCPSession session, ByteBuffer buffer)
/*     */   {
/*     */     for (;;) {
/* 112 */       switch (this.decodeStatus) {
/*     */       case NONE: 
/* 114 */         if (buffer.remaining() < 24) {
/* 115 */           return false;
/*     */         }
/* 117 */         this.decodeStatus = BinaryDecodeStatus.READ_HEADER;
/*     */         
/* 119 */         break;
/*     */       case READ_HEADER: 
/* 121 */         readHeader(buffer);
/* 122 */         break;
/*     */       case READ_EXTRAS: 
/* 124 */         if (readExtras(buffer, this.responseExtrasLength)) {
/* 125 */           this.decodeStatus = BinaryDecodeStatus.READ_KEY;
/*     */         }
/*     */         else
/* 128 */           return false;
/*     */         break;
/*     */       case READ_KEY: 
/* 131 */         if (readKey(buffer, this.responseKeyLength)) {
/* 132 */           this.decodeStatus = BinaryDecodeStatus.READ_VALUE;
/*     */         }
/*     */         else
/* 135 */           return false;
/*     */         break;
/*     */       case READ_VALUE: 
/* 138 */         if ((this.responseStatus == null) || (this.responseStatus == ResponseStatus.NO_ERROR))
/*     */         {
/* 140 */           if (readValue(buffer, this.responseTotalBodyLength, this.responseKeyLength, this.responseExtrasLength))
/*     */           {
/* 142 */             this.decodeStatus = BinaryDecodeStatus.DONE;
/*     */           }
/*     */           else {
/* 145 */             return false;
/*     */           }
/*     */           
/*     */         }
/* 149 */         else if (ByteUtils.stepBuffer(buffer, this.responseTotalBodyLength - this.responseKeyLength - this.responseExtrasLength))
/*     */         {
/*     */ 
/*     */ 
/* 153 */           this.decodeStatus = BinaryDecodeStatus.DONE;
/*     */         }
/*     */         else
/* 156 */           return false;
/*     */         break;
/*     */       case DONE: 
/* 159 */         if (finish()) {
/* 160 */           return true;
/*     */         }
/*     */         
/* 163 */         this.decodeStatus = BinaryDecodeStatus.NONE;
/* 164 */         break;
/*     */       
/*     */       case IGNORE: 
/* 167 */         buffer.reset();
/* 168 */         return true;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   protected boolean finish() {
/* 174 */     if (this.result == null) {
/* 175 */       if (this.responseStatus == ResponseStatus.NO_ERROR) {
/* 176 */         setResult(Boolean.TRUE);
/*     */       } else {
/* 178 */         setResult(Boolean.FALSE);
/*     */       }
/*     */     }
/* 181 */     countDownLatch();
/* 182 */     return true;
/*     */   }
/*     */   
/*     */   protected void readHeader(ByteBuffer buffer) {
/* 186 */     markBuffer(buffer);
/* 187 */     readMagicNumber(buffer);
/* 188 */     if (!readOpCode(buffer)) {
/* 189 */       this.decodeStatus = BinaryDecodeStatus.IGNORE;
/* 190 */       return;
/*     */     }
/* 192 */     readKeyLength(buffer);
/* 193 */     readExtrasLength(buffer);
/* 194 */     readDataType(buffer);
/* 195 */     readStatus(buffer);
/* 196 */     readBodyLength(buffer);
/* 197 */     if (!readOpaque(buffer)) {
/* 198 */       this.decodeStatus = BinaryDecodeStatus.IGNORE;
/* 199 */       return;
/*     */     }
/* 201 */     this.decodeStatus = BinaryDecodeStatus.READ_EXTRAS;
/* 202 */     readCAS(buffer);
/*     */   }
/*     */   
/*     */   private void markBuffer(ByteBuffer buffer)
/*     */   {
/* 207 */     buffer.mark();
/*     */   }
/*     */   
/*     */   protected boolean readOpaque(ByteBuffer buffer) {
/* 211 */     if (this.noreply) {
/* 212 */       int returnOpaque = buffer.getInt();
/* 213 */       if (returnOpaque != this.opaque) {
/* 214 */         return false;
/*     */       }
/*     */     } else {
/* 217 */       ByteUtils.stepBuffer(buffer, 4);
/*     */     }
/* 219 */     return true;
/*     */   }
/*     */   
/*     */   protected long readCAS(ByteBuffer buffer) {
/* 223 */     ByteUtils.stepBuffer(buffer, 8);
/* 224 */     return 0L;
/*     */   }
/*     */   
/*     */   protected boolean readKey(ByteBuffer buffer, int keyLength) {
/* 228 */     return ByteUtils.stepBuffer(buffer, keyLength);
/*     */   }
/*     */   
/*     */   protected boolean readValue(ByteBuffer buffer, int bodyLength, int keyLength, int extrasLength)
/*     */   {
/* 233 */     return ByteUtils.stepBuffer(buffer, bodyLength - keyLength - extrasLength);
/*     */   }
/*     */   
/*     */   protected boolean readExtras(ByteBuffer buffer, int extrasLength)
/*     */   {
/* 238 */     return ByteUtils.stepBuffer(buffer, extrasLength);
/*     */   }
/*     */   
/*     */   private int readBodyLength(ByteBuffer buffer) {
/* 242 */     this.responseTotalBodyLength = buffer.getInt();
/* 243 */     return this.responseTotalBodyLength;
/*     */   }
/*     */   
/*     */   protected void readStatus(ByteBuffer buffer) {
/* 247 */     this.responseStatus = ResponseStatus.parseShort(buffer.getShort());
/* 248 */     switch (this.responseStatus) {
/*     */     case NOT_SUPPORTED: 
/*     */     case UNKNOWN_COMMAND: 
/* 251 */       setException(new UnknownCommandException());
/* 252 */       break;
/*     */     case AUTH_REQUIRED: 
/*     */     case FUTHER_AUTH_REQUIRED: 
/*     */     case VALUE_TOO_BIG: 
/*     */     case INVALID_ARGUMENTS: 
/*     */     case INC_DEC_NON_NUM: 
/*     */     case BELONGS_TO_ANOTHER_SRV: 
/*     */     case AUTH_ERROR: 
/*     */     case OUT_OF_MEMORY: 
/*     */     case INTERNAL_ERROR: 
/*     */     case BUSY: 
/*     */     case TEMP_FAILURE: 
/* 264 */       setException(new MemcachedServerException(this.responseStatus.errorMessage()));
/*     */     }
/*     */     
/*     */   }
/*     */   
/*     */ 
/*     */   public final OpCode getOpCode()
/*     */   {
/* 272 */     return this.opCode;
/*     */   }
/*     */   
/*     */   public final void setOpCode(OpCode opCode) {
/* 276 */     this.opCode = opCode;
/*     */   }
/*     */   
/*     */   public final ResponseStatus getResponseStatus() {
/* 280 */     return this.responseStatus;
/*     */   }
/*     */   
/*     */   public final void setResponseStatus(ResponseStatus responseStatus) {
/* 284 */     this.responseStatus = responseStatus;
/*     */   }
/*     */   
/*     */   private int readKeyLength(ByteBuffer buffer) {
/* 288 */     this.responseKeyLength = buffer.getShort();
/* 289 */     return this.responseKeyLength;
/*     */   }
/*     */   
/*     */   private int readExtrasLength(ByteBuffer buffer) {
/* 293 */     this.responseExtrasLength = buffer.get();
/* 294 */     return this.responseExtrasLength;
/*     */   }
/*     */   
/*     */   private byte readDataType(ByteBuffer buffer) {
/* 298 */     return buffer.get();
/*     */   }
/*     */   
/*     */   protected boolean readOpCode(ByteBuffer buffer) {
/* 302 */     byte op = buffer.get();
/* 303 */     if (op != this.opCode.fieldValue()) {
/* 304 */       if (this.noreply) {
/* 305 */         return false;
/*     */       }
/* 307 */       throw new MemcachedDecodeException("Not a proper " + this.opCode.name() + " response");
/*     */     }
/*     */     
/*     */ 
/* 311 */     return true;
/*     */   }
/*     */   
/*     */   private void readMagicNumber(ByteBuffer buffer) {
/* 315 */     byte magic = buffer.get();
/*     */     
/* 317 */     if (magic != -127) {
/* 318 */       throw new MemcachedDecodeException("Not a proper response");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void encode()
/*     */   {
/* 330 */     CachedData data = null;
/* 331 */     if (this.transcoder != null) {
/* 332 */       data = this.transcoder.encode(this.value);
/*     */     }
/*     */     
/* 335 */     int length = 24 + getKeyLength() + getValueLength(data) + getExtrasLength();
/*     */     
/*     */ 
/* 338 */     this.ioBuffer = IoBuffer.allocate(length);
/* 339 */     fillHeader(data);
/* 340 */     fillExtras(data);
/* 341 */     fillKey();
/* 342 */     fillValue(data);
/*     */     
/* 344 */     this.ioBuffer.flip();
/*     */   }
/*     */   
/*     */   protected void fillValue(CachedData data)
/*     */   {
/* 349 */     this.ioBuffer.put(data.getData());
/*     */   }
/*     */   
/*     */   protected void fillKey() {
/* 353 */     this.ioBuffer.put(this.keyBytes);
/*     */   }
/*     */   
/*     */   protected void fillExtras(CachedData data) {
/* 357 */     this.ioBuffer.putInt(data.getFlag());
/* 358 */     this.ioBuffer.putInt(this.expTime);
/*     */   }
/*     */   
/*     */   protected final void fillHeader(CachedData data) {
/* 362 */     fillMagicNumber();
/* 363 */     fillOpCode();
/* 364 */     fillKeyLength();
/* 365 */     fillExtrasLength();
/* 366 */     fillDataType();
/* 367 */     fillVbucketId();
/* 368 */     fillTotalBodyLength(data);
/* 369 */     fillOpaque();
/* 370 */     fillCAS();
/*     */   }
/*     */   
/*     */   protected void fillCAS()
/*     */   {
/* 375 */     this.ioBuffer.putLong(0L);
/*     */   }
/*     */   
/*     */   private void fillOpaque()
/*     */   {
/* 380 */     if (this.noreply) {
/* 381 */       this.opaque = OpaqueGenerater.getInstance().getNextValue();
/*     */     }
/* 383 */     this.ioBuffer.putInt(this.opaque);
/*     */   }
/*     */   
/*     */   private void fillTotalBodyLength(CachedData data) {
/* 387 */     this.ioBuffer.putInt(getExtrasLength() + getKeyLength() + getValueLength(data));
/*     */   }
/*     */   
/*     */ 
/*     */   private void fillVbucketId()
/*     */   {
/* 393 */     this.ioBuffer.putShort(this.vbucketId);
/*     */   }
/*     */   
/*     */   private void fillDataType()
/*     */   {
/* 398 */     this.ioBuffer.put((byte)0);
/*     */   }
/*     */   
/*     */   private void fillExtrasLength() {
/* 402 */     this.ioBuffer.put(getExtrasLength());
/*     */   }
/*     */   
/*     */   private void fillKeyLength() {
/* 406 */     this.ioBuffer.putShort((short)getKeyLength());
/*     */   }
/*     */   
/*     */   private void fillOpCode() {
/* 410 */     this.ioBuffer.put(this.opCode.fieldValue());
/*     */   }
/*     */   
/*     */   private void fillMagicNumber() {
/* 414 */     this.ioBuffer.put((byte)Byte.MIN_VALUE);
/*     */   }
/*     */   
/*     */   protected int getValueLength(CachedData data) {
/* 418 */     return data.getData().length;
/*     */   }
/*     */   
/*     */   protected int getKeyLength() {
/* 422 */     return this.keyBytes.length;
/*     */   }
/*     */   
/*     */   protected byte getExtrasLength() {
/* 426 */     return 8;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\command\binary\BaseBinaryCommand.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */