/*     */ package org.apache.avro.util;
/*     */ 
/*     */ import java.lang.ref.ReferenceQueue;
/*     */ import java.lang.ref.WeakReference;
/*     */ import java.util.Collection;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class WeakIdentityHashMap<K, V>
/*     */   implements Map<K, V>
/*     */ {
/*  46 */   private final ReferenceQueue<K> queue = new ReferenceQueue();
/*  47 */   private Map<WeakIdentityHashMap<K, V>.IdentityWeakReference, V> backingStore = new HashMap();
/*     */   
/*     */ 
/*     */ 
/*     */   public void clear()
/*     */   {
/*  53 */     this.backingStore.clear();
/*  54 */     reap();
/*     */   }
/*     */   
/*     */   public boolean containsKey(Object key) {
/*  58 */     reap();
/*  59 */     return this.backingStore.containsKey(new IdentityWeakReference(key));
/*     */   }
/*     */   
/*     */   public boolean containsValue(Object value) {
/*  63 */     reap();
/*  64 */     return this.backingStore.containsValue(value);
/*     */   }
/*     */   
/*     */   public Set<Map.Entry<K, V>> entrySet() {
/*  68 */     reap();
/*  69 */     Set<Map.Entry<K, V>> ret = new HashSet();
/*  70 */     for (Map.Entry<WeakIdentityHashMap<K, V>.IdentityWeakReference, V> ref : this.backingStore.entrySet()) {
/*  71 */       final K key = ((IdentityWeakReference)ref.getKey()).get();
/*  72 */       final V value = ref.getValue();
/*  73 */       Map.Entry<K, V> entry = new Map.Entry() {
/*     */         public K getKey() {
/*  75 */           return (K)key;
/*     */         }
/*     */         
/*  78 */         public V getValue() { return (V)value; }
/*     */         
/*     */         public V setValue(V value) {
/*  81 */           throw new UnsupportedOperationException();
/*     */         }
/*  83 */       };
/*  84 */       ret.add(entry);
/*     */     }
/*  86 */     return Collections.unmodifiableSet(ret);
/*     */   }
/*     */   
/*     */   public Set<K> keySet() {
/*  90 */     reap();
/*  91 */     Set<K> ret = new HashSet();
/*  92 */     for (WeakIdentityHashMap<K, V>.IdentityWeakReference ref : this.backingStore.keySet()) {
/*  93 */       ret.add(ref.get());
/*     */     }
/*  95 */     return Collections.unmodifiableSet(ret);
/*     */   }
/*     */   
/*     */   public boolean equals(Object o) {
/*  99 */     return this.backingStore.equals(((WeakIdentityHashMap)o).backingStore);
/*     */   }
/*     */   
/*     */   public V get(Object key) {
/* 103 */     reap();
/* 104 */     return (V)this.backingStore.get(new IdentityWeakReference(key));
/*     */   }
/*     */   
/* 107 */   public V put(K key, V value) { reap();
/* 108 */     return (V)this.backingStore.put(new IdentityWeakReference(key), value);
/*     */   }
/*     */   
/*     */   public int hashCode() {
/* 112 */     reap();
/* 113 */     return this.backingStore.hashCode();
/*     */   }
/*     */   
/* 116 */   public boolean isEmpty() { reap();
/* 117 */     return this.backingStore.isEmpty();
/*     */   }
/*     */   
/* 120 */   public void putAll(Map t) { throw new UnsupportedOperationException(); }
/*     */   
/*     */   public V remove(Object key) {
/* 123 */     reap();
/* 124 */     return (V)this.backingStore.remove(new IdentityWeakReference(key));
/*     */   }
/*     */   
/* 127 */   public int size() { reap();
/* 128 */     return this.backingStore.size();
/*     */   }
/*     */   
/* 131 */   public Collection<V> values() { reap();
/* 132 */     return this.backingStore.values();
/*     */   }
/*     */   
/*     */   private synchronized void reap() {
/* 136 */     Object zombie = this.queue.poll();
/*     */     
/* 138 */     while (zombie != null) {
/* 139 */       WeakIdentityHashMap<K, V>.IdentityWeakReference victim = (IdentityWeakReference)zombie;
/* 140 */       this.backingStore.remove(victim);
/* 141 */       zombie = this.queue.poll();
/*     */     }
/*     */   }
/*     */   
/*     */   class IdentityWeakReference extends WeakReference<K>
/*     */   {
/*     */     int hash;
/*     */     
/*     */     IdentityWeakReference(Object obj) {
/* 150 */       super(WeakIdentityHashMap.this.queue);
/* 151 */       this.hash = System.identityHashCode(obj);
/*     */     }
/*     */     
/*     */     public int hashCode() {
/* 155 */       return this.hash;
/*     */     }
/*     */     
/*     */     public boolean equals(Object o) {
/* 159 */       if (this == o) {
/* 160 */         return true;
/*     */       }
/* 162 */       WeakIdentityHashMap<K, V>.IdentityWeakReference ref = (IdentityWeakReference)o;
/* 163 */       if (get() == ref.get()) {
/* 164 */         return true;
/*     */       }
/* 166 */       return false;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\util\WeakIdentityHashMap.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */