/*     */ package org.codehaus.jackson.map.ser.impl;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.reflect.InvocationTargetException;
/*     */ import java.lang.reflect.Type;
/*     */ import org.codehaus.jackson.JsonGenerationException;
/*     */ import org.codehaus.jackson.JsonGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.ResolvableSerializer;
/*     */ import org.codehaus.jackson.map.SerializerProvider;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.annotate.JacksonStdImpl;
/*     */ import org.codehaus.jackson.map.ser.ArraySerializers.AsArraySerializer;
/*     */ import org.codehaus.jackson.map.ser.ContainerSerializerBase;
/*     */ import org.codehaus.jackson.map.type.ArrayType;
/*     */ import org.codehaus.jackson.node.ObjectNode;
/*     */ import org.codehaus.jackson.schema.JsonSchema;
/*     */ import org.codehaus.jackson.schema.SchemaAware;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ @JacksonStdImpl
/*     */ public class ObjectArraySerializer
/*     */   extends ArraySerializers.AsArraySerializer<Object[]>
/*     */   implements ResolvableSerializer
/*     */ {
/*     */   protected final boolean _staticTyping;
/*     */   protected final JavaType _elementType;
/*     */   protected JsonSerializer<Object> _elementSerializer;
/*     */   protected PropertySerializerMap _dynamicSerializers;
/*     */   
/*     */   @Deprecated
/*     */   public ObjectArraySerializer(JavaType elemType, boolean staticTyping, TypeSerializer vts, BeanProperty property)
/*     */   {
/*  66 */     this(elemType, staticTyping, vts, property, null);
/*     */   }
/*     */   
/*     */ 
/*     */   public ObjectArraySerializer(JavaType elemType, boolean staticTyping, TypeSerializer vts, BeanProperty property, JsonSerializer<Object> elementSerializer)
/*     */   {
/*  72 */     super(Object[].class, vts, property);
/*  73 */     this._elementType = elemType;
/*  74 */     this._staticTyping = staticTyping;
/*  75 */     this._dynamicSerializers = PropertySerializerMap.emptyMap();
/*  76 */     this._elementSerializer = elementSerializer;
/*     */   }
/*     */   
/*     */ 
/*     */   public ContainerSerializerBase<?> _withValueTypeSerializer(TypeSerializer vts)
/*     */   {
/*  82 */     return new ObjectArraySerializer(this._elementType, this._staticTyping, vts, this._property, this._elementSerializer);
/*     */   }
/*     */   
/*     */ 
/*     */   public void serializeContents(Object[] value, JsonGenerator jgen, SerializerProvider provider)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/*  89 */     int len = value.length;
/*  90 */     if (len == 0) {
/*  91 */       return;
/*     */     }
/*  93 */     if (this._elementSerializer != null) {
/*  94 */       serializeContentsUsing(value, jgen, provider, this._elementSerializer);
/*  95 */       return;
/*     */     }
/*  97 */     if (this._valueTypeSerializer != null) {
/*  98 */       serializeTypedContents(value, jgen, provider);
/*  99 */       return;
/*     */     }
/* 101 */     int i = 0;
/* 102 */     Object elem = null;
/*     */     try {
/* 104 */       PropertySerializerMap serializers = this._dynamicSerializers;
/* 105 */       for (; i < len; i++) {
/* 106 */         elem = value[i];
/* 107 */         if (elem == null) {
/* 108 */           provider.defaultSerializeNull(jgen);
/*     */         }
/*     */         else {
/* 111 */           Class<?> cc = elem.getClass();
/* 112 */           JsonSerializer<Object> serializer = serializers.serializerFor(cc);
/* 113 */           if (serializer == null)
/*     */           {
/* 115 */             if (this._elementType.hasGenericTypes()) {
/* 116 */               serializer = _findAndAddDynamic(serializers, this._elementType.forcedNarrowBy(cc), provider);
/*     */             } else {
/* 118 */               serializer = _findAndAddDynamic(serializers, cc, provider);
/*     */             }
/*     */           }
/* 121 */           serializer.serialize(elem, jgen, provider);
/*     */         }
/*     */       }
/* 124 */     } catch (IOException ioe) { throw ioe;
/*     */ 
/*     */ 
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/*     */ 
/* 131 */       Throwable t = e;
/* 132 */       while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 133 */         t = t.getCause();
/*     */       }
/* 135 */       if ((t instanceof Error)) {
/* 136 */         throw ((Error)t);
/*     */       }
/* 138 */       throw JsonMappingException.wrapWithPath(t, elem, i);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void serializeContentsUsing(Object[] value, JsonGenerator jgen, SerializerProvider provider, JsonSerializer<Object> ser)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/* 146 */     int len = value.length;
/* 147 */     TypeSerializer typeSer = this._valueTypeSerializer;
/*     */     
/* 149 */     int i = 0;
/* 150 */     Object elem = null;
/*     */     try {
/* 152 */       for (; i < len; i++) {
/* 153 */         elem = value[i];
/* 154 */         if (elem == null) {
/* 155 */           provider.defaultSerializeNull(jgen);
/*     */ 
/*     */         }
/* 158 */         else if (typeSer == null) {
/* 159 */           ser.serialize(elem, jgen, provider);
/*     */         } else {
/* 161 */           ser.serializeWithType(elem, jgen, provider, typeSer);
/*     */         }
/*     */       }
/*     */     } catch (IOException ioe) {
/* 165 */       throw ioe;
/*     */     } catch (Exception e) {
/* 167 */       Throwable t = e;
/* 168 */       while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 169 */         t = t.getCause();
/*     */       }
/* 171 */       if ((t instanceof Error)) {
/* 172 */         throw ((Error)t);
/*     */       }
/* 174 */       throw JsonMappingException.wrapWithPath(t, elem, i);
/*     */     }
/*     */   }
/*     */   
/*     */   public void serializeTypedContents(Object[] value, JsonGenerator jgen, SerializerProvider provider)
/*     */     throws IOException, JsonGenerationException
/*     */   {
/* 181 */     int len = value.length;
/* 182 */     TypeSerializer typeSer = this._valueTypeSerializer;
/* 183 */     int i = 0;
/* 184 */     Object elem = null;
/*     */     try {
/* 186 */       PropertySerializerMap serializers = this._dynamicSerializers;
/* 187 */       for (; i < len; i++) {
/* 188 */         elem = value[i];
/* 189 */         if (elem == null) {
/* 190 */           provider.defaultSerializeNull(jgen);
/*     */         }
/*     */         else {
/* 193 */           Class<?> cc = elem.getClass();
/* 194 */           JsonSerializer<Object> serializer = serializers.serializerFor(cc);
/* 195 */           if (serializer == null) {
/* 196 */             serializer = _findAndAddDynamic(serializers, cc, provider);
/*     */           }
/* 198 */           serializer.serializeWithType(elem, jgen, provider, typeSer);
/*     */         }
/*     */       }
/* 201 */     } catch (IOException ioe) { throw ioe;
/*     */     } catch (Exception e) {
/* 203 */       Throwable t = e;
/* 204 */       while (((t instanceof InvocationTargetException)) && (t.getCause() != null)) {
/* 205 */         t = t.getCause();
/*     */       }
/* 207 */       if ((t instanceof Error)) {
/* 208 */         throw ((Error)t);
/*     */       }
/* 210 */       throw JsonMappingException.wrapWithPath(t, elem, i);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public JsonNode getSchema(SerializerProvider provider, Type typeHint)
/*     */     throws JsonMappingException
/*     */   {
/* 218 */     ObjectNode o = createSchemaNode("array", true);
/* 219 */     if (typeHint != null) {
/* 220 */       JavaType javaType = provider.constructType(typeHint);
/* 221 */       if (javaType.isArrayType()) {
/* 222 */         Class<?> componentType = ((ArrayType)javaType).getContentType().getRawClass();
/*     */         
/* 224 */         if (componentType == Object.class) {
/* 225 */           o.put("items", JsonSchema.getDefaultSchemaNode());
/*     */         } else {
/* 227 */           JsonSerializer<Object> ser = provider.findValueSerializer(componentType, this._property);
/* 228 */           JsonNode schemaNode = (ser instanceof SchemaAware) ? ((SchemaAware)ser).getSchema(provider, null) : JsonSchema.getDefaultSchemaNode();
/*     */           
/*     */ 
/* 231 */           o.put("items", schemaNode);
/*     */         }
/*     */       }
/*     */     }
/* 235 */     return o;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void resolve(SerializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/* 245 */     if ((this._staticTyping) && (this._elementSerializer == null)) {
/* 246 */       this._elementSerializer = provider.findValueSerializer(this._elementType, this._property);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map, Class<?> type, SerializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/* 256 */     PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSerializer(type, provider, this._property);
/*     */     
/* 258 */     if (map != result.map) {
/* 259 */       this._dynamicSerializers = result.map;
/*     */     }
/* 261 */     return result.serializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final JsonSerializer<Object> _findAndAddDynamic(PropertySerializerMap map, JavaType type, SerializerProvider provider)
/*     */     throws JsonMappingException
/*     */   {
/* 270 */     PropertySerializerMap.SerializerAndMapResult result = map.findAndAddSerializer(type, provider, this._property);
/*     */     
/* 272 */     if (map != result.map) {
/* 273 */       this._dynamicSerializers = result.map;
/*     */     }
/* 275 */     return result.serializer;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\impl\ObjectArraySerializer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */