/*      */ package org.codehaus.janino;
/*      */ 
/*      */ import java.io.DataOutputStream;
/*      */ import java.io.IOException;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Arrays;
/*      */ import java.util.Collections;
/*      */ import java.util.HashMap;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import org.codehaus.janino.util.ClassFile;
/*      */ import org.codehaus.janino.util.ClassFile.AttributeInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantFieldrefInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantInterfaceMethodrefInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantMethodrefInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantNameAndTypeInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantPoolInfo;
/*      */ import org.codehaus.janino.util.ClassFile.ConstantUtf8Info;
/*      */ import org.codehaus.janino.util.ClassFile.LineNumberTableAttribute;
/*      */ import org.codehaus.janino.util.ClassFile.LineNumberTableAttribute.Entry;
/*      */ import org.codehaus.janino.util.ClassFile.LocalVariableTableAttribute;
/*      */ import org.codehaus.janino.util.ClassFile.LocalVariableTableAttribute.Entry;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class CodeContext
/*      */ {
/*      */   private static final boolean DEBUG = false;
/*      */   private static final int INITIAL_SIZE = 128;
/*      */   private static final byte UNEXAMINED = -1;
/*      */   private static final byte INVALID_OFFSET = -2;
/*      */   private static final int MAX_STACK_SIZE = 254;
/*      */   private ClassFile classFile;
/*      */   private short maxStack;
/*      */   private short maxLocals;
/*      */   private byte[] code;
/*      */   private Offset beginning;
/*      */   private Inserter end;
/*      */   private Inserter currentInserter;
/*      */   private List exceptionTableEntries;
/*   67 */   private List allLocalVars = new ArrayList();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   73 */   private List scopedVars = new ArrayList();
/*   74 */   private short nextLocalVariableSlot = 0;
/*      */   
/*   76 */   private final List relocatables = new ArrayList();
/*      */   
/*      */ 
/*      */ 
/*      */   public CodeContext(ClassFile classFile)
/*      */   {
/*   82 */     this.classFile = classFile;
/*      */     
/*   84 */     this.maxStack = 0;
/*   85 */     this.maxLocals = 0;
/*   86 */     this.code = new byte[''];
/*   87 */     this.beginning = new Offset();
/*   88 */     this.end = new Inserter();
/*   89 */     this.currentInserter = this.end;
/*   90 */     this.exceptionTableEntries = new ArrayList();
/*      */     
/*   92 */     this.beginning.offset = 0;
/*   93 */     this.end.offset = 0;
/*   94 */     this.beginning.next = this.end;
/*   95 */     this.end.prev = this.beginning;
/*      */   }
/*      */   
/*      */   public ClassFile getClassFile() {
/*   99 */     return this.classFile;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short allocateLocalVariable(short size)
/*      */   {
/*  120 */     return allocateLocalVariable(size, null, null).getSlotIndex();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.LocalVariableSlot allocateLocalVariable(short size, String name, IClass type)
/*      */   {
/*  142 */     List currentVars = null;
/*      */     
/*  144 */     if (this.scopedVars.size() == 0) {
/*  145 */       throw new Error("saveLocalVariables must be called first");
/*      */     }
/*  147 */     currentVars = (List)this.scopedVars.get(this.scopedVars.size() - 1);
/*      */     
/*      */ 
/*  150 */     Java.LocalVariableSlot slot = new Java.LocalVariableSlot(name, this.nextLocalVariableSlot, type);
/*      */     
/*  152 */     if (slot.getName() != null) {
/*  153 */       slot.setStart(newOffset());
/*      */     }
/*      */     
/*  156 */     this.nextLocalVariableSlot = ((short)(this.nextLocalVariableSlot + size));
/*  157 */     currentVars.add(slot);
/*  158 */     this.allLocalVars.add(slot);
/*      */     
/*  160 */     if (this.nextLocalVariableSlot > this.maxLocals) {
/*  161 */       this.maxLocals = this.nextLocalVariableSlot;
/*      */     }
/*      */     
/*  164 */     return slot;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public List saveLocalVariables()
/*      */   {
/*  174 */     List l = new ArrayList();
/*  175 */     this.scopedVars.add(l);
/*      */     
/*  177 */     return l;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void restoreLocalVariables()
/*      */   {
/*  189 */     Iterator iter = ((List)this.scopedVars.remove(this.scopedVars.size() - 1)).iterator();
/*      */     
/*  191 */     while (iter.hasNext()) {
/*  192 */       Java.LocalVariableSlot slot = (Java.LocalVariableSlot)iter.next();
/*      */       
/*  194 */       if (slot.getName() != null) {
/*  195 */         slot.setEnd(newOffset());
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void storeCodeAttributeBody(DataOutputStream dos, short lineNumberTableAttributeNameIndex, short localVariableTableAttributeNameIndex)
/*      */     throws IOException
/*      */   {
/*  211 */     dos.writeShort(this.maxStack);
/*  212 */     dos.writeShort(this.maxLocals);
/*  213 */     dos.writeInt(this.end.offset);
/*  214 */     dos.write(this.code, 0, this.end.offset);
/*  215 */     dos.writeShort(this.exceptionTableEntries.size());
/*  216 */     for (int i = 0; i < this.exceptionTableEntries.size(); i++) {
/*  217 */       ExceptionTableEntry exceptionTableEntry = (ExceptionTableEntry)this.exceptionTableEntries.get(i);
/*  218 */       dos.writeShort(exceptionTableEntry.startPC.offset);
/*  219 */       dos.writeShort(exceptionTableEntry.endPC.offset);
/*  220 */       dos.writeShort(exceptionTableEntry.handlerPC.offset);
/*  221 */       dos.writeShort(exceptionTableEntry.catchType);
/*      */     }
/*      */     
/*  224 */     List attributes = new ArrayList();
/*      */     
/*      */ 
/*  227 */     if (lineNumberTableAttributeNameIndex != 0) {
/*  228 */       List lnt = new ArrayList();
/*  229 */       for (Offset o = this.beginning; o != null; o = o.next) {
/*  230 */         if ((o instanceof LineNumberOffset)) {
/*  231 */           lnt.add(new ClassFile.LineNumberTableAttribute.Entry(o.offset, ((LineNumberOffset)o).lineNumber));
/*      */         }
/*      */       }
/*  234 */       ClassFile.LineNumberTableAttribute.Entry[] lnte = (ClassFile.LineNumberTableAttribute.Entry[])lnt.toArray(new ClassFile.LineNumberTableAttribute.Entry[lnt.size()]);
/*      */       
/*      */ 
/*  237 */       attributes.add(new ClassFile.LineNumberTableAttribute(lineNumberTableAttributeNameIndex, lnte));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  243 */     if (localVariableTableAttributeNameIndex != 0) {
/*  244 */       ClassFile.AttributeInfo ai = storeLocalVariableTable(dos, localVariableTableAttributeNameIndex);
/*      */       
/*  246 */       if (ai != null) { attributes.add(ai);
/*      */       }
/*      */     }
/*  249 */     dos.writeShort(attributes.size());
/*  250 */     for (Iterator it = attributes.iterator(); it.hasNext();) {
/*  251 */       ClassFile.AttributeInfo attribute = (ClassFile.AttributeInfo)it.next();
/*  252 */       attribute.store(dos);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected ClassFile.AttributeInfo storeLocalVariableTable(DataOutputStream dos, short localVariableTableAttributeNameIndex)
/*      */   {
/*  260 */     ClassFile cf = getClassFile();
/*  261 */     Iterator iter = getAllLocalVars().iterator();
/*  262 */     List entryList = new ArrayList();
/*      */     
/*  264 */     while (iter.hasNext()) {
/*  265 */       Java.LocalVariableSlot slot = (Java.LocalVariableSlot)iter.next();
/*      */       
/*  267 */       if (slot.getName() != null) {
/*  268 */         String typeName = slot.getType().getDescriptor();
/*  269 */         short classSlot = cf.addConstantUtf8Info(typeName);
/*  270 */         short varNameSlot = cf.addConstantUtf8Info(slot.getName());
/*      */         
/*      */ 
/*      */ 
/*  274 */         ClassFile.LocalVariableTableAttribute.Entry entry = new ClassFile.LocalVariableTableAttribute.Entry((short)slot.getStart().offset, (short)(slot.getEnd().offset - slot.getStart().offset), varNameSlot, classSlot, slot.getSlotIndex());
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  281 */         entryList.add(entry);
/*      */       }
/*      */     }
/*      */     
/*  285 */     if (entryList.size() > 0) {
/*  286 */       Object entries = entryList.toArray(new ClassFile.LocalVariableTableAttribute.Entry[entryList.size()]);
/*      */       
/*  288 */       return new ClassFile.LocalVariableTableAttribute(localVariableTableAttributeNameIndex, (ClassFile.LocalVariableTableAttribute.Entry[])entries);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  293 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void flowAnalysis(String functionName)
/*      */   {
/*  307 */     short[] stackSizes = new short[this.end.offset];
/*  308 */     Arrays.fill(stackSizes, (short)-1);
/*      */     
/*      */ 
/*  311 */     flowAnalysis(functionName, this.code, this.end.offset, 0, (short)0, stackSizes);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  321 */     int analyzedExceptionHandlers = 0;
/*  322 */     while (analyzedExceptionHandlers != this.exceptionTableEntries.size()) {
/*  323 */       for (int i = 0; i < this.exceptionTableEntries.size(); i++) {
/*  324 */         ExceptionTableEntry exceptionTableEntry = (ExceptionTableEntry)this.exceptionTableEntries.get(i);
/*  325 */         if (stackSizes[exceptionTableEntry.startPC.offset] != -1) {
/*  326 */           flowAnalysis(functionName, this.code, this.end.offset, exceptionTableEntry.handlerPC.offset, (short)(stackSizes[exceptionTableEntry.startPC.offset] + 1), stackSizes);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  334 */           analyzedExceptionHandlers++;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  340 */     this.maxStack = 0;
/*  341 */     for (int i = 0; i < stackSizes.length; i++) {
/*  342 */       short ss = stackSizes[i];
/*  343 */       if (ss == -1)
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*  348 */         throw new JaninoRuntimeException(functionName + ": Unexamined code at offset " + i);
/*      */       }
/*      */       
/*  351 */       if (ss > this.maxStack) { this.maxStack = ss;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void flowAnalysis(String functionName, byte[] code, int codeSize, int offset, short stackSize, short[] stackSizes)
/*      */   {
/*      */     for (;;)
/*      */     {
/*  367 */       if ((offset < 0) || (offset >= codeSize)) {
/*  368 */         throw new JaninoRuntimeException(functionName + ": Offset out of range");
/*      */       }
/*      */       
/*      */ 
/*  372 */       int css = stackSizes[offset];
/*  373 */       if (css == stackSize) return;
/*  374 */       if (css == -2) throw new JaninoRuntimeException(functionName + ": Invalid offset");
/*  375 */       if (css != -1)
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  388 */         throw new JaninoRuntimeException(functionName + ": Operand stack inconsistent at offset " + offset + ": Previous size " + css + ", now " + stackSize);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  399 */       stackSizes[offset] = stackSize;
/*      */       
/*      */ 
/*  402 */       byte opcode = code[offset];
/*  403 */       int operandOffset = offset + 1;
/*      */       short props;
/*  405 */       short props; if (opcode == -60) {
/*  406 */         opcode = code[(operandOffset++)];
/*  407 */         props = Opcode.WIDE_OPCODE_PROPERTIES[(0xFF & opcode)];
/*      */       } else {
/*  409 */         props = Opcode.OPCODE_PROPERTIES[(0xFF & opcode)];
/*      */       }
/*  411 */       if (props == -1) {
/*  412 */         throw new JaninoRuntimeException(functionName + ": Invalid opcode " + (0xFF & opcode) + " at offset " + offset);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  421 */       switch (props & 0x1F)
/*      */       {
/*      */       case 0: 
/*      */       case 1: 
/*      */       case 2: 
/*      */       case 3: 
/*      */       case 4: 
/*      */       case 5: 
/*      */       case 6: 
/*  430 */         stackSize = (short)(stackSize + ((props & 0x1F) - 4));
/*  431 */         break;
/*      */       
/*      */       case 7: 
/*  434 */         stackSize = 0;
/*  435 */         break;
/*      */       
/*      */       case 9: 
/*  438 */         stackSize = (short)(stackSize - 1);
/*      */       
/*      */       case 10: 
/*  441 */         stackSize = (short)(stackSize + determineFieldSize((short)extract16BitValue(0, operandOffset, code)));
/*      */         
/*      */ 
/*  444 */         break;
/*      */       
/*      */       case 11: 
/*  447 */         stackSize = (short)(stackSize - 1);
/*      */       
/*      */       case 12: 
/*  450 */         stackSize = (short)(stackSize - determineFieldSize((short)extract16BitValue(0, operandOffset, code)));
/*      */         
/*      */ 
/*  453 */         break;
/*      */       
/*      */       case 13: 
/*      */       case 14: 
/*      */       case 16: 
/*  458 */         stackSize = (short)(stackSize - 1);
/*      */       
/*      */       case 15: 
/*  461 */         stackSize = (short)(stackSize - determineArgumentsSize((short)extract16BitValue(0, operandOffset, code)));
/*      */         
/*      */ 
/*  464 */         break;
/*      */       
/*      */       case 18: 
/*  467 */         stackSize = (short)(stackSize - (code[(operandOffset + 2)] - 1));
/*  468 */         break;
/*      */       case 8: case 17: 
/*      */       default: 
/*  471 */         throw new JaninoRuntimeException(functionName + ": Invalid stack delta");
/*      */       }
/*      */       
/*  474 */       if (stackSize < 0) {
/*  475 */         String msg = this.classFile.getThisClassName() + '.' + functionName + ": Operand stack underrun at offset " + offset;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  486 */         throw new JaninoRuntimeException(msg);
/*      */       }
/*      */       
/*      */ 
/*  490 */       if (stackSize > 254) {
/*  491 */         String msg = this.classFile.getThisClassName() + '.' + functionName + ": Operand stack overflow at offset " + offset;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  502 */         throw new JaninoRuntimeException(msg);
/*      */       }
/*      */       
/*      */ 
/*  506 */       switch (props & 0x1E0)
/*      */       {
/*      */       case 0: 
/*      */         break;
/*      */       
/*      */ 
/*      */       case 32: 
/*      */       case 64: 
/*      */       case 128: 
/*      */       case 192: 
/*  516 */         operandOffset++;
/*  517 */         break;
/*      */       
/*      */       case 96: 
/*      */       case 160: 
/*      */       case 224: 
/*  522 */         operandOffset += 2;
/*  523 */         break;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       case 256: 
/*  532 */         flowAnalysis(functionName, code, codeSize, extract16BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  539 */         operandOffset += 2;
/*  540 */         break;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       case 384: 
/*  549 */         int targetOffset = extract16BitValue(offset, operandOffset, code);
/*  550 */         operandOffset += 2;
/*  551 */         if (stackSizes[targetOffset] == -1) {
/*  552 */           flowAnalysis(functionName, code, codeSize, targetOffset, (short)(stackSize + 1), stackSizes);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         break;
/*      */       case 288: 
/*  563 */         flowAnalysis(functionName, code, codeSize, extract32BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  569 */         operandOffset += 4;
/*  570 */         break;
/*      */       
/*      */       case 320: 
/*  573 */         while ((operandOffset & 0x3) != 0) operandOffset++;
/*  574 */         flowAnalysis(functionName, code, codeSize, extract32BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  580 */         operandOffset += 4;
/*      */         
/*  582 */         int npairs = extract32BitValue(0, operandOffset, code);
/*  583 */         operandOffset += 4;
/*      */         
/*  585 */         for (int i = 0; i < npairs; i++) {
/*  586 */           operandOffset += 4;
/*  587 */           flowAnalysis(functionName, code, codeSize, extract32BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  593 */           operandOffset += 4;
/*      */         }
/*  595 */         break;
/*      */       
/*      */       case 352: 
/*  598 */         while ((operandOffset & 0x3) != 0) operandOffset++;
/*  599 */         flowAnalysis(functionName, code, codeSize, extract32BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  605 */         operandOffset += 4;
/*  606 */         int low = extract32BitValue(offset, operandOffset, code);
/*  607 */         operandOffset += 4;
/*  608 */         int hi = extract32BitValue(offset, operandOffset, code);
/*  609 */         operandOffset += 4;
/*  610 */         for (int i = low; i <= hi; i++) {
/*  611 */           flowAnalysis(functionName, code, codeSize, extract32BitValue(offset, operandOffset, code), stackSize, stackSizes);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  617 */           operandOffset += 4;
/*      */         }
/*  619 */         break;
/*      */       
/*      */       default: 
/*  622 */         throw new JaninoRuntimeException(functionName + ": Invalid OP1");
/*      */       }
/*      */       
/*  625 */       switch (props & 0x600)
/*      */       {
/*      */       case 0: 
/*      */         break;
/*      */       
/*      */ 
/*      */       case 512: 
/*  632 */         operandOffset++;
/*  633 */         break;
/*      */       
/*      */       case 1024: 
/*  636 */         operandOffset += 2;
/*  637 */         break;
/*      */       
/*      */       default: 
/*  640 */         throw new JaninoRuntimeException(functionName + ": Invalid OP2");
/*      */       }
/*      */       
/*  643 */       switch (props & 0x800)
/*      */       {
/*      */       case 0: 
/*      */         break;
/*      */       
/*      */ 
/*      */       case 2048: 
/*  650 */         operandOffset++;
/*  651 */         break;
/*      */       
/*      */       default: 
/*  654 */         throw new JaninoRuntimeException(functionName + ": Invalid OP3");
/*      */       }
/*      */       
/*  657 */       Arrays.fill(stackSizes, offset + 1, operandOffset, (short)-2);
/*      */       
/*  659 */       if ((props & 0x8000) != 0) return;
/*  660 */       offset = operandOffset;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int extract16BitValue(int bias, int offset, byte[] code)
/*      */   {
/*  673 */     int res = bias + ((code[offset] << 8) + (code[(offset + 1)] & 0xFF));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  682 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int extract32BitValue(int bias, int offset, byte[] code)
/*      */   {
/*  694 */     int res = bias + ((code[offset] << 24) + ((0xFF & code[(offset + 1)]) << 16) + ((0xFF & code[(offset + 2)]) << 8) + (0xFF & code[(offset + 3)]));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  711 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void fixUpAndRelocate()
/*      */   {
/*      */     do
/*      */     {
/*  724 */       fixUp();
/*  725 */     } while (!relocate());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void fixUp()
/*      */   {
/*  732 */     for (Offset o = this.beginning; o != this.end; o = o.next) {
/*  733 */       if ((o instanceof FixUp)) { ((FixUp)o).fixUp();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private boolean relocate()
/*      */   {
/*  743 */     boolean finished = true;
/*  744 */     for (int i = 0; i < this.relocatables.size(); i++)
/*      */     {
/*      */ 
/*  747 */       boolean part = ((Relocatable)this.relocatables.get(i)).relocate();
/*  748 */       finished = (finished) && (part);
/*      */     }
/*  750 */     return finished;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private int determineFieldSize(short idx)
/*      */   {
/*  757 */     ClassFile.ConstantFieldrefInfo cfi = (ClassFile.ConstantFieldrefInfo)this.classFile.getConstantPoolInfo(idx);
/*      */     
/*      */ 
/*  760 */     ClassFile.ConstantNameAndTypeInfo cnati = (ClassFile.ConstantNameAndTypeInfo)this.classFile.getConstantPoolInfo(cfi.getNameAndTypeIndex());
/*      */     
/*      */ 
/*  763 */     ClassFile.ConstantUtf8Info cui = (ClassFile.ConstantUtf8Info)this.classFile.getConstantPoolInfo(cnati.getDescriptorIndex());
/*      */     
/*      */ 
/*  766 */     return Descriptor.size(cui.getString());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private int determineArgumentsSize(short idx)
/*      */   {
/*  774 */     ClassFile.ConstantPoolInfo cpi = this.classFile.getConstantPoolInfo(idx);
/*  775 */     ClassFile.ConstantNameAndTypeInfo nat = (ClassFile.ConstantNameAndTypeInfo)this.classFile.getConstantPoolInfo((cpi instanceof ClassFile.ConstantInterfaceMethodrefInfo) ? ((ClassFile.ConstantInterfaceMethodrefInfo)cpi).getNameAndTypeIndex() : ((ClassFile.ConstantMethodrefInfo)cpi).getNameAndTypeIndex());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  780 */     ClassFile.ConstantUtf8Info cui = (ClassFile.ConstantUtf8Info)this.classFile.getConstantPoolInfo(nat.getDescriptorIndex());
/*      */     
/*      */ 
/*  783 */     String desc = cui.getString();
/*      */     
/*  785 */     if (desc.charAt(0) != '(') throw new JaninoRuntimeException("Method descriptor does not start with \"(\"");
/*  786 */     int i = 1;
/*  787 */     int res = 0;
/*      */     for (;;) {
/*  789 */       switch (desc.charAt(i++)) {
/*      */       case ')': 
/*  791 */         return res - Descriptor.size(desc.substring(i));
/*      */       case 'B': case 'C': case 'F': case 'I': case 'S': case 'Z': 
/*  793 */         res++;
/*  794 */         break;
/*      */       case 'D': case 'J': 
/*  796 */         res += 2;
/*  797 */         break;
/*      */       case '[': 
/*  799 */         res++;
/*  800 */         while (desc.charAt(i) == '[') i++;
/*  801 */         if ("BCFISZDJ".indexOf(desc.charAt(i)) == -1) break label273; i++; } }
/*  802 */     label273: if (desc.charAt(i) != 'L') throw new JaninoRuntimeException("Invalid char after \"[\"");
/*  803 */     i++;
/*  804 */     while (desc.charAt(i++) != ';') {}
/*      */     
/*      */ 
/*  807 */     res++;
/*  808 */     while (desc.charAt(i++) != ';') {}
/*      */     
/*      */ 
/*  811 */     throw new JaninoRuntimeException("Invalid method descriptor");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void write(short lineNumber, byte[] b)
/*      */   {
/*  824 */     if (b.length == 0) { return;
/*      */     }
/*  826 */     int ico = this.currentInserter.offset;
/*  827 */     makeSpace(lineNumber, b.length);
/*  828 */     System.arraycopy(b, 0, this.code, ico, b.length);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void write(short lineNumber, byte b1)
/*      */   {
/*  842 */     int ico = this.currentInserter.offset;
/*  843 */     makeSpace(lineNumber, 1);
/*  844 */     this.code[ico] = b1;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void write(short lineNumber, byte b1, byte b2)
/*      */   {
/*  859 */     int ico = this.currentInserter.offset;
/*  860 */     makeSpace(lineNumber, 2);
/*  861 */     this.code[(ico++)] = b1;
/*  862 */     this.code[ico] = b2;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void write(short lineNumber, byte b1, byte b2, byte b3)
/*      */   {
/*  878 */     int ico = this.currentInserter.offset;
/*  879 */     makeSpace(lineNumber, 3);
/*  880 */     this.code[(ico++)] = b1;
/*  881 */     this.code[(ico++)] = b2;
/*  882 */     this.code[ico] = b3;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void write(short lineNumber, byte b1, byte b2, byte b3, byte b4)
/*      */   {
/*  899 */     int ico = this.currentInserter.offset;
/*  900 */     makeSpace(lineNumber, 4);
/*  901 */     this.code[(ico++)] = b1;
/*  902 */     this.code[(ico++)] = b2;
/*  903 */     this.code[(ico++)] = b3;
/*  904 */     this.code[ico] = b4;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void makeSpace(short lineNumber, int size)
/*      */   {
/*  915 */     if (size == 0) { return;
/*      */     }
/*      */     
/*  918 */     if (lineNumber != -1)
/*      */     {
/*  920 */       for (Offset o = this.currentInserter.prev; o != this.beginning; o = o.prev)
/*  921 */         if ((o instanceof LineNumberOffset)) {
/*  922 */           if (((LineNumberOffset)o).lineNumber != lineNumber)
/*      */             break;
/*      */           break label115;
/*      */         }
/*  926 */       LineNumberOffset lno = new LineNumberOffset(this.currentInserter.offset, lineNumber);
/*  927 */       lno.prev = this.currentInserter.prev;
/*  928 */       lno.next = this.currentInserter;
/*  929 */       this.currentInserter.prev.next = lno;
/*  930 */       this.currentInserter.prev = lno;
/*      */     }
/*      */     label115:
/*  933 */     int ico = this.currentInserter.offset;
/*  934 */     if (this.end.offset + size <= this.code.length)
/*      */     {
/*  936 */       if (ico != this.end.offset) {
/*  937 */         System.arraycopy(this.code, ico, this.code, ico + size, this.end.offset - ico);
/*      */       }
/*      */     } else {
/*  940 */       byte[] oldCode = this.code;
/*      */       
/*  942 */       int newSize = Math.max(Math.min(oldCode.length * 2, 65535), oldCode.length + size);
/*  943 */       if (newSize > 65535) {
/*  944 */         throw new JaninoRuntimeException("Code attribute in class \"" + this.classFile.getThisClassName() + "\" grows beyond 64 KB");
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  950 */       this.code = new byte[newSize];
/*  951 */       System.arraycopy(oldCode, 0, this.code, 0, ico);
/*  952 */       System.arraycopy(oldCode, ico, this.code, ico + size, this.end.offset - ico);
/*      */     }
/*  954 */     Arrays.fill(this.code, ico, ico + size, (byte)0);
/*  955 */     for (Offset o = this.currentInserter; o != null; o = o.next) { o.offset += size;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public void writeShort(short lineNumber, int v)
/*      */   {
/*  962 */     write(lineNumber, (byte)(v >> 8), (byte)v);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void writeBranch(short lineNumber, int opcode, Offset dst)
/*      */   {
/*  969 */     this.relocatables.add(new Branch(opcode, dst));
/*  970 */     write(lineNumber, (byte)opcode, (byte)-1, (byte)-1); }
/*      */   
/*      */   private class Branch extends CodeContext.Relocatable { private boolean expanded;
/*      */     
/*  974 */     public Branch(int opcode, CodeContext.Offset destination) { super(null);
/*  975 */       this.opcode = opcode;
/*  976 */       this.source = CodeContext.this.newInserter();
/*  977 */       this.destination = destination;
/*  978 */       if ((opcode == -55) || (opcode == -56))
/*      */       {
/*  980 */         this.expanded = true;
/*      */       } else {
/*  982 */         this.expanded = false;
/*      */       }
/*      */     }
/*      */     
/*      */     public boolean relocate() {
/*  987 */       if (this.destination.offset == -1) {
/*  988 */         throw new JaninoRuntimeException("Cannot relocate branch to unset destination offset");
/*      */       }
/*  990 */       int offset = this.destination.offset - this.source.offset;
/*      */       
/*  992 */       if ((!this.expanded) && ((offset > 32767) || (offset < 32768)))
/*      */       {
/*      */ 
/*  995 */         int pos = this.source.offset;
/*  996 */         CodeContext.this.pushInserter(this.source);
/*      */         
/*      */ 
/*  999 */         CodeContext.this.makeSpace((short)-1, this.opcode == -88 ? 2 : this.opcode == -89 ? 2 : 5);
/*      */         
/*      */ 
/*      */ 
/* 1003 */         CodeContext.this.popInserter();
/* 1004 */         this.source.offset = pos;
/* 1005 */         this.expanded = true;
/* 1006 */         return false;
/*      */       }
/*      */       byte[] ba;
/*      */       byte[] ba;
/* 1010 */       if (!this.expanded)
/*      */       {
/* 1012 */         ba = new byte[] { (byte)this.opcode, (byte)(offset >> 8), (byte)offset };
/*      */       } else { byte[] ba;
/* 1014 */         if ((this.opcode == -89) || (this.opcode == -88)) {
/* 1015 */           ba = new byte[] { (byte)(this.opcode + 33), (byte)(offset >> 24), (byte)(offset >> 16), (byte)(offset >> 8), (byte)offset };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1027 */           offset -= 3;
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1033 */           ba = new byte[] { CodeContext.invertBranchOpcode((byte)this.opcode), 0, 8, -56, (byte)(offset >> 24), (byte)(offset >> 16), (byte)(offset >> 8), (byte)offset };
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1045 */       System.arraycopy(ba, 0, CodeContext.this.code, this.source.offset, ba.length);
/* 1046 */       return true;
/*      */     }
/*      */     
/*      */ 
/*      */     private final int opcode;
/*      */     
/*      */     private final CodeContext.Inserter source;
/*      */     
/*      */     private final CodeContext.Offset destination;
/*      */   }
/*      */   
/*      */   private static byte invertBranchOpcode(byte branchOpcode)
/*      */   {
/* 1059 */     return ((Byte)BRANCH_OPCODE_INVERSION.get(new Byte(branchOpcode))).byteValue();
/*      */   }
/*      */   
/*      */ 
/* 1063 */   private static final Map BRANCH_OPCODE_INVERSION = ;
/*      */   
/* 1065 */   private static Map createBranchOpcodeInversion() { Map m = new HashMap();
/* 1066 */     m.put(new Byte((byte)-91), new Byte((byte)-90));
/* 1067 */     m.put(new Byte((byte)-90), new Byte((byte)-91));
/* 1068 */     m.put(new Byte((byte)-97), new Byte((byte)-96));
/* 1069 */     m.put(new Byte((byte)-96), new Byte((byte)-97));
/* 1070 */     m.put(new Byte((byte)-94), new Byte((byte)-95));
/* 1071 */     m.put(new Byte((byte)-95), new Byte((byte)-94));
/* 1072 */     m.put(new Byte((byte)-93), new Byte((byte)-92));
/* 1073 */     m.put(new Byte((byte)-92), new Byte((byte)-93));
/* 1074 */     m.put(new Byte((byte)-103), new Byte((byte)-102));
/* 1075 */     m.put(new Byte((byte)-102), new Byte((byte)-103));
/* 1076 */     m.put(new Byte((byte)-100), new Byte((byte)-101));
/* 1077 */     m.put(new Byte((byte)-101), new Byte((byte)-100));
/* 1078 */     m.put(new Byte((byte)-99), new Byte((byte)-98));
/* 1079 */     m.put(new Byte((byte)-98), new Byte((byte)-99));
/* 1080 */     m.put(new Byte((byte)-58), new Byte((byte)-57));
/* 1081 */     m.put(new Byte((byte)-57), new Byte((byte)-58));
/* 1082 */     return Collections.unmodifiableMap(m);
/*      */   }
/*      */   
/*      */   public void writeOffset(short lineNumber, Offset src, Offset dst) {
/* 1086 */     this.relocatables.add(new OffsetBranch(newOffset(), src, dst));
/* 1087 */     write(lineNumber, (byte)-1, (byte)-1, (byte)-1, (byte)-1); }
/*      */   
/*      */   private class OffsetBranch extends CodeContext.Relocatable { private final CodeContext.Offset where;
/*      */     
/* 1091 */     public OffsetBranch(CodeContext.Offset where, CodeContext.Offset source, CodeContext.Offset destination) { super(null);
/* 1092 */       this.where = where;
/* 1093 */       this.source = source;
/* 1094 */       this.destination = destination;
/*      */     }
/*      */     
/* 1097 */     public boolean relocate() { if ((this.source.offset == -1) || (this.destination.offset == -1))
/*      */       {
/*      */ 
/* 1100 */         throw new JaninoRuntimeException("Cannot relocate offset branch to unset destination offset"); }
/* 1101 */       int offset = this.destination.offset - this.source.offset;
/* 1102 */       byte[] ba = { (byte)(offset >> 24), (byte)(offset >> 16), (byte)(offset >> 8), (byte)offset };
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1108 */       System.arraycopy(ba, 0, CodeContext.this.code, this.where.offset, 4);
/* 1109 */       return true;
/*      */     }
/*      */     
/*      */     private final CodeContext.Offset source;
/*      */     private final CodeContext.Offset destination; }
/*      */   
/* 1115 */   public Offset newOffset() { Offset o = new Offset();
/* 1116 */     o.set();
/* 1117 */     return o;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Inserter newInserter()
/*      */   {
/* 1131 */     Inserter i = new Inserter();
/* 1132 */     i.set();
/* 1133 */     return i;
/*      */   }
/*      */   
/*      */   public Inserter currentInserter() {
/* 1137 */     return this.currentInserter;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void pushInserter(Inserter ins)
/*      */   {
/* 1145 */     if (ins.nextInserter != null) throw new JaninoRuntimeException("An Inserter can only be pushed once at a time");
/* 1146 */     ins.nextInserter = this.currentInserter;
/* 1147 */     this.currentInserter = ins;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public void popInserter()
/*      */   {
/* 1155 */     Inserter ni = this.currentInserter.nextInserter;
/* 1156 */     if (ni == null) throw new JaninoRuntimeException("Code inserter stack underflow");
/* 1157 */     this.currentInserter.nextInserter = null;
/* 1158 */     this.currentInserter = ni;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public class Offset
/*      */   {
/* 1169 */     int offset = -1;
/* 1170 */     Offset prev = null; Offset next = null;
/*      */     
/*      */     static final int UNSET = -1;
/*      */     
/*      */     public Offset() {}
/*      */     
/*      */     public void set()
/*      */     {
/* 1178 */       if (this.offset != -1) { throw new JaninoRuntimeException("Cannot \"set()\" Offset more than once");
/*      */       }
/* 1180 */       this.offset = CodeContext.this.currentInserter.offset;
/* 1181 */       this.prev = CodeContext.this.currentInserter.prev;
/* 1182 */       this.next = CodeContext.this.currentInserter;
/*      */       
/* 1184 */       this.prev.next = this;
/* 1185 */       this.next.prev = this; }
/*      */     
/* 1187 */     public final CodeContext getCodeContext() { return CodeContext.this; }
/*      */     
/*      */     public String toString() {
/* 1190 */       return CodeContext.this.classFile.getThisClassName() + ": " + this.offset;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void addExceptionTableEntry(Offset startPC, Offset endPC, Offset handlerPC, String catchTypeFD)
/*      */   {
/* 1207 */     this.exceptionTableEntries.add(new ExceptionTableEntry(startPC, endPC, handlerPC, catchTypeFD == null ? 0 : this.classFile.addConstantClassInfo(catchTypeFD)));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static class ExceptionTableEntry
/*      */   {
/*      */     final CodeContext.Offset startPC;
/*      */     
/*      */ 
/*      */     final CodeContext.Offset endPC;
/*      */     
/*      */     final CodeContext.Offset handlerPC;
/*      */     
/*      */     final short catchType;
/*      */     
/*      */ 
/*      */     ExceptionTableEntry(CodeContext.Offset startPC, CodeContext.Offset endPC, CodeContext.Offset handlerPC, short catchType)
/*      */     {
/* 1226 */       this.startPC = startPC;
/* 1227 */       this.endPC = endPC;
/* 1228 */       this.handlerPC = handlerPC;
/* 1229 */       this.catchType = catchType;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public class Inserter
/*      */     extends CodeContext.Offset
/*      */   {
/* 1239 */     public Inserter() { super(); }
/* 1240 */     private Inserter nextInserter = null;
/*      */   }
/*      */   
/*      */   public class LineNumberOffset extends CodeContext.Offset { private final int lineNumber;
/*      */     
/* 1245 */     public LineNumberOffset(int offset, int lineNumber) { super();
/* 1246 */       this.lineNumber = lineNumber;
/* 1247 */       this.offset = offset; } }
/*      */   
/*      */   private abstract class Relocatable { private Relocatable() {}
/*      */     public abstract boolean relocate();
/* 1251 */     Relocatable(CodeContext.1 x1) { this(); }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public List getAllLocalVars()
/*      */   {
/* 1273 */     return this.allLocalVars;
/*      */   }
/*      */   
/*      */   public static abstract interface FixUp
/*      */   {
/*      */     public abstract void fixUp();
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\CodeContext.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */