/*     */ package org.codehaus.janino.tools;
/*     */ 
/*     */ import java.io.File;
/*     */ import java.io.FilenameFilter;
/*     */ import java.io.IOException;
/*     */ import java.io.PrintStream;
/*     */ import java.io.StringReader;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.commons.compiler.CompilerFactoryFactory;
/*     */ import org.codehaus.commons.compiler.IClassBodyEvaluator;
/*     */ import org.codehaus.commons.compiler.ICompilerFactory;
/*     */ import org.codehaus.commons.compiler.IExpressionEvaluator;
/*     */ import org.codehaus.commons.compiler.IScriptEvaluator;
/*     */ import org.codehaus.janino.Descriptor;
/*     */ import org.codehaus.janino.ExpressionEvaluator;
/*     */ import org.codehaus.janino.IClass;
/*     */ import org.codehaus.janino.IClass.IInvocable;
/*     */ import org.codehaus.janino.IClass.IMethod;
/*     */ import org.codehaus.janino.IClassLoader;
/*     */ import org.codehaus.janino.Java.CompilationUnit;
/*     */ import org.codehaus.janino.Java.ConstructorInvocation;
/*     */ import org.codehaus.janino.Java.Invocation;
/*     */ import org.codehaus.janino.Java.Located;
/*     */ import org.codehaus.janino.Java.MethodInvocation;
/*     */ import org.codehaus.janino.Java.NewAnonymousClassInstance;
/*     */ import org.codehaus.janino.Java.NewClassInstance;
/*     */ import org.codehaus.janino.Java.SuperclassMethodInvocation;
/*     */ import org.codehaus.janino.Parser;
/*     */ import org.codehaus.janino.Scanner;
/*     */ import org.codehaus.janino.Scanner.Token;
/*     */ import org.codehaus.janino.UnitCompiler;
/*     */ import org.codehaus.janino.util.Benchmark;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ import org.codehaus.janino.util.StringPattern;
/*     */ import org.codehaus.janino.util.Traverser;
/*     */ import org.codehaus.janino.util.enumerator.Enumerator;
/*     */ import org.codehaus.janino.util.iterator.DirectoryIterator;
/*     */ import org.codehaus.janino.util.resource.PathResourceFinder;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JGrep
/*     */ {
/*     */   private static final boolean DEBUG = false;
/*  82 */   private List parsedCompilationUnits = new ArrayList();
/*     */   
/*     */ 
/*     */ 
/*     */   public static void main(String[] args)
/*     */   {
/*  88 */     int idx = 0;
/*     */     
/*  90 */     StringPattern[] directoryNamePatterns = StringPattern.PATTERNS_ALL;
/*  91 */     StringPattern[] fileNamePatterns = { new StringPattern("*.java") };
/*  92 */     File[] classPath = { new File(".") };
/*  93 */     File[] optionalExtDirs = null;
/*  94 */     File[] optionalBootClassPath = null;
/*  95 */     String optionalCharacterEncoding = null;
/*  96 */     boolean verbose = false;
/*  98 */     for (; 
/*  98 */         idx < args.length; idx++) {
/*  99 */       String arg = args[idx];
/* 100 */       if (arg.charAt(0) != '-') break;
/* 101 */       if (arg.equals("-dirs")) {
/* 102 */         directoryNamePatterns = StringPattern.parseCombinedPattern(args[(++idx)]);
/*     */       }
/* 104 */       else if (arg.equals("-files")) {
/* 105 */         fileNamePatterns = StringPattern.parseCombinedPattern(args[(++idx)]);
/*     */       }
/* 107 */       else if (arg.equals("-classpath")) {
/* 108 */         classPath = PathResourceFinder.parsePath(args[(++idx)]);
/*     */       }
/* 110 */       else if (arg.equals("-extdirs")) {
/* 111 */         optionalExtDirs = PathResourceFinder.parsePath(args[(++idx)]);
/*     */       }
/* 113 */       else if (arg.equals("-bootclasspath")) {
/* 114 */         optionalBootClassPath = PathResourceFinder.parsePath(args[(++idx)]);
/*     */       }
/* 116 */       else if (arg.equals("-encoding")) {
/* 117 */         optionalCharacterEncoding = args[(++idx)];
/*     */       }
/* 119 */       else if (arg.equals("-verbose")) {
/* 120 */         verbose = true;
/*     */       }
/* 122 */       else if (arg.equals("-help")) {
/* 123 */         for (int j = 0; j < USAGE.length; j++) System.out.println(USAGE[j]);
/* 124 */         System.exit(1);
/*     */       }
/*     */       else {
/* 127 */         System.err.println("Unexpected command-line argument \"" + arg + "\", try \"-help\".");
/* 128 */         System.exit(1);
/* 129 */         return;
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 136 */     int first = idx;
/* 137 */     while ((idx < args.length) && (args[idx].charAt(0) != '-')) idx++;
/* 138 */     if (idx == first) {
/* 139 */       System.err.println("No <directory-path>es given, try \"-help\".");
/* 140 */       System.exit(1);
/* 141 */       return;
/*     */     }
/* 143 */     File[] rootDirectories = new File[idx - first];
/* 144 */     for (int i = first; i < idx; i++) { rootDirectories[(i - first)] = new File(args[i]);
/*     */     }
/*     */     
/*     */ 
/* 148 */     JGrep jGrep = new JGrep(classPath, optionalExtDirs, optionalBootClassPath, optionalCharacterEncoding, verbose);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 156 */     List mits = new ArrayList();
/* 157 */     for (; idx < args.length; idx++) {
/* 158 */       String arg = args[idx];
/* 159 */       if (arg.equals("-method-invocation")) {
/*     */         MethodInvocationTarget mit;
/*     */         try {
/* 162 */           mit = parseMethodInvocationPattern(args[(++idx)]);
/*     */         } catch (Exception ex) {
/* 164 */           System.err.println("Parsing method invocation pattern \"" + args[idx] + "\": " + ex.getMessage());
/* 165 */           System.exit(1);
/* 166 */           return;
/*     */         }
/* 168 */         while (idx < args.length - 1) {
/* 169 */           arg = args[(idx + 1)];
/* 170 */           if (arg.startsWith("predicate:")) {
/* 171 */             String predicateExpression = arg.substring(10);
/*     */             try {
/* 173 */               IExpressionEvaluator ee = new ExpressionEvaluator();
/* 174 */               ee.setClassName(JGrep.class.getName() + "PE");
/* 175 */               mit.predicates.add((MethodInvocationPredicate)ee.createFastEvaluator(predicateExpression, class$org$codehaus$janino$tools$JGrep$MethodInvocationPredicate, new String[] { "uc", "invocation", "method" }));
/*     */ 
/*     */             }
/*     */             catch (Exception ex)
/*     */             {
/*     */ 
/* 181 */               System.err.println("Compiling predicate expression \"" + predicateExpression + "\": " + ex.getMessage());
/*     */               
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 187 */               System.exit(1);
/* 188 */               return;
/*     */             }
/*     */           } else {
/* 191 */             if (!arg.startsWith("action:")) break;
/* 192 */             String action = arg.substring(7);
/*     */             try {
/* 194 */               mit.actions.add(Action.getMethodInvocationAction(action));
/*     */             } catch (Exception ex) {
/* 196 */               System.err.println("Compiling method invocation action \"" + action + "\": " + ex.getMessage());
/*     */               
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 202 */               System.exit(1);
/* 203 */               return;
/*     */             }
/*     */           }
/*     */           
/*     */ 
/*     */ 
/* 209 */           idx++;
/*     */         }
/* 211 */         mits.add(mit);
/*     */       }
/*     */       else {
/* 214 */         System.err.println("Unexpected command-line argument \"" + arg + "\", try \"-help\".");
/* 215 */         System.exit(1);
/* 216 */         return;
/*     */       }
/*     */     }
/*     */     
/*     */     try
/*     */     {
/* 222 */       jGrep.jGrep(rootDirectories, directoryNamePatterns, fileNamePatterns, mits);
/*     */ 
/*     */ 
/*     */     }
/*     */     catch (Exception e)
/*     */     {
/*     */ 
/* 229 */       System.err.println(e.toString());
/* 230 */       System.exit(1);
/*     */     }
/*     */   }
/*     */   
/*     */   private static final class Action extends Enumerator {
/* 235 */     private Action(String name) { super(); }
/*     */     
/*     */     static JGrep.MethodInvocationAction getMethodInvocationAction(String action) throws CompileException {
/* 238 */       if ("print-location-and-match".equals(action)) {
/* 239 */         new JGrep.MethodInvocationAction() {
/*     */           public void execute(UnitCompiler uc, Java.Invocation invocation, IClass.IMethod method) {
/* 241 */             System.out.println(invocation.getLocation() + ": " + method);
/*     */           }
/*     */         };
/*     */       }
/* 245 */       if ("print-location".equals(action)) {
/* 246 */         new JGrep.MethodInvocationAction() {
/*     */           public void execute(UnitCompiler uc, Java.Invocation invocation, IClass.IMethod method) {
/* 248 */             System.out.println(invocation.getLocation());
/*     */           }
/*     */         };
/*     */       }
/*     */       ICompilerFactory cf;
/*     */       try
/*     */       {
/* 255 */         cf = CompilerFactoryFactory.getDefaultCompilerFactory();
/*     */       } catch (Exception e) {
/* 257 */         e.printStackTrace();
/* 258 */         throw new RuntimeException(e.getMessage());
/*     */       }
/* 260 */       return (JGrep.MethodInvocationAction)cf.newScriptEvaluator().createFastEvaluator(action, JGrep.class$org$codehaus$janino$tools$JGrep$MethodInvocationAction, new String[] { "uc", "invocation", "method" });
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static MethodInvocationTarget parseMethodInvocationPattern(String mip)
/*     */     throws CompileException, IOException
/*     */   {
/* 272 */     MethodInvocationTarget mit = new MethodInvocationTarget(null);
/* 273 */     Scanner scanner = new Scanner(null, new StringReader(mip));
/* 274 */     Parser parser = new Parser(scanner);
/*     */     for (;;)
/*     */     {
/* 277 */       String s = readIdentifierPattern(parser);
/* 278 */       if (parser.peekOperator("(")) {
/* 279 */         mit.methodNamePattern = s;
/* 280 */         parser.eatToken();
/* 281 */         List l = new ArrayList();
/* 282 */         if (!parser.peekOperator(")")) {
/*     */           for (;;) {
/* 284 */             l.add(readIdentifierPattern(parser));
/* 285 */             if (parser.peekOperator(")")) break;
/* 286 */             parser.readOperator(",");
/*     */           }
/*     */         }
/* 289 */         mit.optionalArgumentTypeNamePatterns = ((String[])l.toArray(new String[l.size()]));
/* 290 */         return mit;
/*     */       }
/* 292 */       if (parser.peekOperator(".")) {
/* 293 */         if (mit.optionalClassNamePattern == null) {
/* 294 */           mit.optionalClassNamePattern = s;
/*     */         }
/*     */         else {
/* 297 */           MethodInvocationTarget tmp173_172 = mit;tmp173_172.optionalClassNamePattern = (tmp173_172.optionalClassNamePattern + '.' + s);
/*     */         }
/* 299 */         parser.eatToken();
/*     */       }
/* 301 */       else if (scanner.peek().isEOF()) {
/* 302 */         mit.methodNamePattern = s;
/* 303 */         return mit;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private static String readIdentifierPattern(Parser p) throws CompileException, IOException {
/* 309 */     StringBuffer sb = new StringBuffer();
/* 310 */     if (p.peekOperator("*")) {
/* 311 */       sb.append('*');
/* 312 */       p.eatToken();
/*     */     }
/*     */     else {
/* 315 */       sb.append(p.readIdentifier());
/*     */     }
/*     */     for (;;) {
/* 318 */       if (p.peekOperator("*")) {
/* 319 */         sb.append('*');
/* 320 */         p.eatToken();
/*     */       } else {
/* 322 */         if (!p.peekIdentifier()) break;
/* 323 */         sb.append(p.readIdentifier());
/*     */       }
/*     */     }
/* 326 */     return sb.toString();
/*     */   }
/*     */   
/*     */   private static class MethodInvocationTarget {
/* 330 */     MethodInvocationTarget(JGrep.1 x0) { this(); }
/* 331 */     String optionalClassNamePattern = null;
/* 332 */     String methodNamePattern = null;
/* 333 */     String[] optionalArgumentTypeNamePatterns = null;
/* 334 */     List predicates = new ArrayList();
/* 335 */     List actions = new ArrayList();
/*     */     
/*     */     void apply(UnitCompiler uc, Java.Invocation invocation, IClass.IMethod method) throws CompileException {
/* 338 */       if ((this.optionalClassNamePattern != null) && 
/* 339 */         (!JGrep.typeMatches(this.optionalClassNamePattern, Descriptor.toClassName(method.getDeclaringIClass().getDescriptor()))))
/*     */       {
/*     */ 
/* 342 */         return;
/*     */       }
/* 344 */       if (!new StringPattern(this.methodNamePattern).matches(method.getName())) return;
/* 345 */       IClass[] fpts = method.getParameterTypes();
/* 346 */       if (this.optionalArgumentTypeNamePatterns != null) {
/* 347 */         String[] atnps = this.optionalArgumentTypeNamePatterns;
/* 348 */         if (atnps.length != fpts.length) return;
/* 349 */         for (int i = 0; i < atnps.length; i++) {
/* 350 */           if (new StringPattern(atnps[i]).matches(Descriptor.toClassName(fpts[i].getDescriptor()))) {}
/*     */         }
/*     */       }
/* 353 */       for (Iterator it = this.predicates.iterator(); it.hasNext();) {
/* 354 */         JGrep.MethodInvocationPredicate mip = (JGrep.MethodInvocationPredicate)it.next();
/*     */         try {
/* 356 */           if (!mip.evaluate(uc, invocation, method)) return;
/*     */         } catch (Exception ex) {
/* 358 */           return;
/*     */         }
/*     */       }
/* 361 */       for (Iterator it = this.actions.iterator(); it.hasNext();) {
/* 362 */         JGrep.MethodInvocationAction mia = (JGrep.MethodInvocationAction)it.next();
/*     */         try {
/* 364 */           mia.execute(uc, invocation, method);
/*     */         }
/*     */         catch (Exception ex) {}
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     private MethodInvocationTarget() {}
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   static boolean typeMatches(String pattern, String typeName)
/*     */   {
/* 379 */     return new StringPattern(pattern).matches(pattern.indexOf('.') == -1 ? typeName.substring(typeName.lastIndexOf('.') + 1) : typeName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 385 */   private static final String[] USAGE = { "Usage:", "", "  java org.codehaus.janino.tools.JGrep [ <option> ... ] <root-dir> ... <pattern> ...", "  java org.codehaus.janino.tools.JGrep -help", "", "Reads a set of compilation units from the files in the <root-dir>s and their", "subdirectories and searches them for specific Java[TM] constructs, e.g.", "invocations of a particular method.", "", "Supported <option>s are ('cp' is a 'combined pattern, like '*.java-*Generated*'):", "  -dirs <dir-cp>             Ignore subdirectories which don't match", "  -files <file-cp>           Include only matching files (default is '*.java')", "  -classpath <classpath>", "  -extdirs <classpath>", "  -bootclasspath <classpath>", "  -encoding <encoding>", "  -verbose", "", "Supported <pattern>s are:", "  -method-invocation <method-pattern> [ predicate:<predicate-expression> | action:<action-script> ] ...", "<method-pattern> is ('<ip>' is an 'identifier pattern' like '*foo*'):", "  -method-invocation <method-ip>", "  -method-invocation <simple-class-ip>.<method-ip>", "  -method-invocation <fully-qualified-class-ip>.<method-ip>", "  -method-invocation <method-ip>([<parameter-ip>[,<parameter-ip>]...])", "", "<predicate-expression> is a Java[TM] expression with the following signature:", "  boolean evaluate(UnitCompiler uc, Java.Invocation invocation, IClass.IMethod method)", "", "<action-script> is either", "  print-location-and-match", "  print-location", ", or a Java[TM] script (method body) with the following signature:", "  void execute(UnitCompiler uc, Java.Invocation invocation, IClass.IMethod method)" };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final IClassLoader iClassLoader;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final String optionalCharacterEncoding;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private final Benchmark benchmark;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JGrep(File[] classPath, File[] optionalExtDirs, File[] optionalBootClassPath, String optionalCharacterEncoding, boolean verbose)
/*     */   {
/* 433 */     this(IClassLoader.createJavacLikePathIClassLoader(optionalBootClassPath, optionalExtDirs, classPath), optionalCharacterEncoding, verbose);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 443 */     this.benchmark.report("*** JGrep - search Java(TM) source files for specific language constructs");
/* 444 */     this.benchmark.report("*** For more information visit http://janino.codehaus.org");
/* 445 */     this.benchmark.report("Class path", classPath);
/* 446 */     this.benchmark.report("Ext dirs", optionalExtDirs);
/* 447 */     this.benchmark.report("Boot class path", optionalBootClassPath);
/* 448 */     this.benchmark.report("Character encoding", optionalCharacterEncoding);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public JGrep(IClassLoader iClassLoader, String optionalCharacterEncoding, boolean verbose)
/*     */   {
/* 456 */     this.iClassLoader = new JGrepIClassLoader(iClassLoader);
/* 457 */     this.optionalCharacterEncoding = optionalCharacterEncoding;
/* 458 */     this.benchmark = new Benchmark(verbose);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void jGrep(File[] rootDirectories, StringPattern[] directoryNamePatterns, StringPattern[] fileNamePatterns, List methodInvocationTargets)
/*     */     throws CompileException, IOException
/*     */   {
/* 467 */     this.benchmark.report("Root dirs", rootDirectories);
/* 468 */     this.benchmark.report("Directory name patterns", directoryNamePatterns);
/* 469 */     this.benchmark.report("File name patterns", fileNamePatterns);
/*     */     
/* 471 */     jGrep(DirectoryIterator.traverseDirectories(rootDirectories, new FilenameFilter()
/*     */     
/*     */ 
/*     */ 
/* 475 */       new FilenameFilter {
/*     */         private final StringPattern[] val$directoryNamePatterns;
/*     */         
/*     */ 
/* 475 */         public boolean accept(File dir, String name) { return StringPattern.matches(this.val$directoryNamePatterns, name); } }, new FilenameFilter()
/*     */       {
/*     */         private final StringPattern[] val$fileNamePatterns;
/*     */         
/*     */ 
/* 480 */         public boolean accept(File dir, String name) { return StringPattern.matches(this.val$fileNamePatterns, name); } }), methodInvocationTargets);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void jGrep(Iterator sourceFilesIterator, List methodInvocationTargets)
/*     */     throws CompileException, IOException
/*     */   {
/* 492 */     this.benchmark.beginReporting();
/* 493 */     int sourceFileCount = 0;
/*     */     
/*     */     try
/*     */     {
/* 497 */       while (sourceFilesIterator.hasNext()) {
/* 498 */         File sourceFile = (File)sourceFilesIterator.next();
/* 499 */         UnitCompiler uc = new UnitCompiler(parseCompilationUnit(sourceFile, this.optionalCharacterEncoding), this.iClassLoader);
/*     */         
/*     */ 
/*     */ 
/* 503 */         this.parsedCompilationUnits.add(uc);
/* 504 */         sourceFileCount++;
/*     */       }
/*     */     } finally {
/* 507 */       this.benchmark.endReporting("Parsed " + sourceFileCount + " source file(s)");
/*     */     }
/*     */     
/*     */ 
/* 511 */     this.benchmark.beginReporting();
/*     */     try {
/* 513 */       for (it = this.parsedCompilationUnits.iterator(); it.hasNext();) {
/* 514 */         UnitCompiler uc = (UnitCompiler)it.next();
/* 515 */         this.benchmark.beginReporting("Grepping \"" + uc.compilationUnit.optionalFileName + "\"");
/*     */         
/*     */ 
/*     */ 
/*     */         try
/*     */         {
/* 521 */           new Traverser() {
/*     */             private final UnitCompiler val$uc;
/*     */             private final List val$methodInvocationTargets;
/*     */             
/*     */             public void traverseMethodInvocation(Java.MethodInvocation mi) {
/* 526 */               try { match(mi, this.val$uc.findIMethod(mi));
/*     */               } catch (CompileException ex) {
/* 528 */                 throw new JGrep.1UCE(JGrep.this, ex);
/*     */               }
/* 530 */               super.traverseMethodInvocation(mi);
/*     */             }
/*     */             
/*     */             public void traverseSuperclassMethodInvocation(Java.SuperclassMethodInvocation scmi)
/*     */             {
/*     */               try {
/* 536 */                 match(scmi, this.val$uc.findIMethod(scmi));
/*     */               } catch (CompileException ex) {
/* 538 */                 throw new JGrep.1UCE(JGrep.this, ex);
/*     */               }
/* 540 */               super.traverseSuperclassMethodInvocation(scmi);
/*     */             }
/*     */             
/*     */ 
/*     */             public void traverseNewClassInstance(Java.NewClassInstance nci)
/*     */             {
/* 546 */               super.traverseNewClassInstance(nci);
/*     */             }
/*     */             
/*     */ 
/*     */             public void traverseNewAnonymousClassInstance(Java.NewAnonymousClassInstance naci)
/*     */             {
/* 552 */               super.traverseNewAnonymousClassInstance(naci);
/*     */             }
/*     */             
/*     */ 
/*     */             public void traverseConstructorInvocation(Java.ConstructorInvocation ci)
/*     */             {
/* 558 */               super.traverseConstructorInvocation(ci);
/*     */             }
/*     */             
/*     */             private void match(Java.Invocation invocation, IClass.IMethod method) throws CompileException {
/* 562 */               for (Iterator it2 = this.val$methodInvocationTargets.iterator(); it2.hasNext();) {
/* 563 */                 JGrep.MethodInvocationTarget mit = (JGrep.MethodInvocationTarget)it2.next();
/* 564 */                 mit.apply(this.val$uc, invocation, method); } } }.traverseCompilationUnit(uc.compilationUnit);
/*     */ 
/*     */         }
/*     */         catch (1UCE uce)
/*     */         {
/* 569 */           throw uce.ce;
/*     */         } finally {
/* 571 */           this.benchmark.endReporting();
/*     */         }
/*     */       }
/*     */     } finally { Iterator it;
/* 575 */       this.benchmark.endReporting("Traversed " + sourceFileCount + " compilation units");
/*     */     }
/*     */   }
/*     */   
/*     */   /* Error */
/*     */   private Java.CompilationUnit parseCompilationUnit(File sourceFile, String optionalCharacterEncoding)
/*     */     throws CompileException, IOException
/*     */   {
/*     */     // Byte code:
/*     */     //   0: new 156	java/io/BufferedInputStream
/*     */     //   3: dup
/*     */     //   4: new 157	java/io/FileInputStream
/*     */     //   7: dup
/*     */     //   8: aload_1
/*     */     //   9: invokespecial 158	java/io/FileInputStream:<init>	(Ljava/io/File;)V
/*     */     //   12: invokespecial 159	java/io/BufferedInputStream:<init>	(Ljava/io/InputStream;)V
/*     */     //   15: astore_3
/*     */     //   16: new 81	org/codehaus/janino/Parser
/*     */     //   19: dup
/*     */     //   20: new 77	org/codehaus/janino/Scanner
/*     */     //   23: dup
/*     */     //   24: aload_1
/*     */     //   25: invokevirtual 160	java/io/File:getPath	()Ljava/lang/String;
/*     */     //   28: aload_3
/*     */     //   29: aload_2
/*     */     //   30: invokespecial 161	org/codehaus/janino/Scanner:<init>	(Ljava/lang/String;Ljava/io/InputStream;Ljava/lang/String;)V
/*     */     //   33: invokespecial 82	org/codehaus/janino/Parser:<init>	(Lorg/codehaus/janino/Scanner;)V
/*     */     //   36: astore 4
/*     */     //   38: aload_0
/*     */     //   39: getfield 107	org/codehaus/janino/tools/JGrep:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   42: new 31	java/lang/StringBuffer
/*     */     //   45: dup
/*     */     //   46: invokespecial 32	java/lang/StringBuffer:<init>	()V
/*     */     //   49: ldc -94
/*     */     //   51: invokevirtual 34	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
/*     */     //   54: aload_1
/*     */     //   55: invokevirtual 163	java/lang/StringBuffer:append	(Ljava/lang/Object;)Ljava/lang/StringBuffer;
/*     */     //   58: ldc -110
/*     */     //   60: invokevirtual 34	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
/*     */     //   63: invokevirtual 36	java/lang/StringBuffer:toString	()Ljava/lang/String;
/*     */     //   66: invokevirtual 147	org/codehaus/janino/util/Benchmark:beginReporting	(Ljava/lang/String;)V
/*     */     //   69: aload 4
/*     */     //   71: invokevirtual 164	org/codehaus/janino/Parser:parseCompilationUnit	()Lorg/codehaus/janino/Java$CompilationUnit;
/*     */     //   74: astore 5
/*     */     //   76: aload_0
/*     */     //   77: getfield 107	org/codehaus/janino/tools/JGrep:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   80: invokevirtual 151	org/codehaus/janino/util/Benchmark:endReporting	()V
/*     */     //   83: aload_3
/*     */     //   84: invokevirtual 165	java/io/InputStream:close	()V
/*     */     //   87: goto +5 -> 92
/*     */     //   90: astore 6
/*     */     //   92: aload 5
/*     */     //   94: areturn
/*     */     //   95: astore 7
/*     */     //   97: aload_0
/*     */     //   98: getfield 107	org/codehaus/janino/tools/JGrep:benchmark	Lorg/codehaus/janino/util/Benchmark;
/*     */     //   101: invokevirtual 151	org/codehaus/janino/util/Benchmark:endReporting	()V
/*     */     //   104: aload 7
/*     */     //   106: athrow
/*     */     //   107: astore 8
/*     */     //   109: aload_3
/*     */     //   110: invokevirtual 165	java/io/InputStream:close	()V
/*     */     //   113: goto +5 -> 118
/*     */     //   116: astore 9
/*     */     //   118: aload 8
/*     */     //   120: athrow
/*     */     // Line number table:
/*     */     //   Java source line #589	-> byte code offset #0
/*     */     //   Java source line #591	-> byte code offset #16
/*     */     //   Java source line #593	-> byte code offset #38
/*     */     //   Java source line #595	-> byte code offset #69
/*     */     //   Java source line #597	-> byte code offset #76
/*     */     //   Java source line #600	-> byte code offset #83
/*     */     //   Java source line #597	-> byte code offset #95
/*     */     //   Java source line #600	-> byte code offset #107
/*     */     // Local variable table:
/*     */     //   start	length	slot	name	signature
/*     */     //   0	121	0	this	JGrep
/*     */     //   0	121	1	sourceFile	File
/*     */     //   0	121	2	optionalCharacterEncoding	String
/*     */     //   15	95	3	is	java.io.InputStream
/*     */     //   36	34	4	parser	Parser
/*     */     //   74	19	5	localCompilationUnit	Java.CompilationUnit
/*     */     //   90	3	6	ex	IOException
/*     */     //   95	10	7	localObject1	Object
/*     */     //   107	12	8	localObject2	Object
/*     */     //   116	3	9	ex	IOException
/*     */     // Exception table:
/*     */     //   from	to	target	type
/*     */     //   83	87	90	java/io/IOException
/*     */     //   69	76	95	finally
/*     */     //   95	97	95	finally
/*     */     //   16	83	107	finally
/*     */     //   95	109	107	finally
/*     */     //   109	113	116	java/io/IOException
/*     */   }
/*     */   
/*     */   public static File getClassFile(String className, File sourceFile, File optionalDestinationDirectory)
/*     */   {
/* 621 */     if (optionalDestinationDirectory != null) {
/* 622 */       return new File(optionalDestinationDirectory, ClassFile.getClassFileResourceName(className));
/*     */     }
/* 624 */     int idx = className.lastIndexOf('.');
/* 625 */     return new File(sourceFile.getParentFile(), ClassFile.getClassFileResourceName(className.substring(idx + 1)));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static abstract interface MethodInvocationPredicate
/*     */   {
/*     */     public abstract boolean evaluate(UnitCompiler paramUnitCompiler, Java.Invocation paramInvocation, IClass.IMethod paramIMethod)
/*     */       throws Exception;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public static abstract interface MethodInvocationAction
/*     */   {
/*     */     public abstract void execute(UnitCompiler paramUnitCompiler, Java.Invocation paramInvocation, IClass.IMethod paramIMethod)
/*     */       throws Exception;
/*     */   }
/*     */   
/*     */ 
/*     */   private class JGrepIClassLoader
/*     */     extends IClassLoader
/*     */   {
/*     */     public JGrepIClassLoader(IClassLoader optionalParentIClassLoader)
/*     */     {
/* 650 */       super();
/* 651 */       super.postConstruct();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected IClass findIClass(String type)
/*     */     {
/* 661 */       String className = Descriptor.toClassName(type);
/*     */       
/*     */ 
/*     */ 
/* 665 */       if (className.startsWith("java.")) { return null;
/*     */       }
/*     */       
/* 668 */       for (int i = 0; i < JGrep.this.parsedCompilationUnits.size(); i++) {
/* 669 */         UnitCompiler uc = (UnitCompiler)JGrep.this.parsedCompilationUnits.get(i);
/* 670 */         IClass res = uc.findClass(className);
/* 671 */         if (res != null) {
/* 672 */           defineIClass(res);
/* 673 */           return res;
/*     */         }
/*     */       }
/* 676 */       return null;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\tools\JGrep.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */