/*     */ package org.github.jamm;
/*     */ 
/*     */ import java.lang.reflect.Field;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.Set;
/*     */ import java.util.Stack;
/*     */ import java.util.concurrent.Callable;
/*     */ 
/*     */ public class MemoryMeter
/*     */ {
/*     */   private static java.lang.instrument.Instrumentation instrumentation;
/*     */   private final Callable<Set<Object>> trackerProvider;
/*     */   private final boolean includeFullBufferSize;
/*     */   
/*     */   public static void premain(String options, java.lang.instrument.Instrumentation inst)
/*     */   {
/*  17 */     instrumentation = inst;
/*     */   }
/*     */   
/*     */   public static boolean isInitialized() {
/*  21 */     return instrumentation != null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public MemoryMeter()
/*     */   {
/*  28 */     this(new Callable()
/*     */     {
/*     */ 
/*     */       public Set<Object> call()
/*     */         throws Exception {
/*  33 */         return java.util.Collections.newSetFromMap(new java.util.IdentityHashMap()); } }, true);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private MemoryMeter(Callable<Set<Object>> trackerProvider, boolean includeFullBufferSize)
/*     */   {
/*  43 */     this.trackerProvider = trackerProvider;
/*  44 */     this.includeFullBufferSize = includeFullBufferSize;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public MemoryMeter withTrackerProvider(Callable<Set<Object>> trackerProvider)
/*     */   {
/*  52 */     return new MemoryMeter(trackerProvider, this.includeFullBufferSize);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public MemoryMeter omitSharedBufferOverhead()
/*     */   {
/*  61 */     return new MemoryMeter(this.trackerProvider, false);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public long measure(Object object)
/*     */   {
/*  69 */     if (instrumentation == null) {
/*  70 */       throw new IllegalStateException("Instrumentation is not set; Jamm must be set as -javaagent");
/*     */     }
/*  72 */     return instrumentation.getObjectSize(object);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public long measureDeep(Object object)
/*     */   {
/*  80 */     if (object == null) {
/*  81 */       throw new NullPointerException();
/*     */     }
/*     */     Set<Object> tracker;
/*     */     try
/*     */     {
/*  86 */       tracker = (Set)this.trackerProvider.call();
/*     */     }
/*     */     catch (Exception e) {
/*  89 */       throw new RuntimeException(e);
/*     */     }
/*     */     
/*  92 */     tracker.add(object);
/*     */     
/*     */ 
/*  95 */     Stack<Object> stack = new Stack();
/*  96 */     stack.push(object);
/*     */     
/*  98 */     long total = 0L;
/*  99 */     while (!stack.isEmpty()) {
/* 100 */       Object current = stack.pop();
/* 101 */       assert (current != null);
/* 102 */       total += measure(current);
/*     */       
/* 104 */       if ((current instanceof Object[])) {
/* 105 */         addArrayChildren((Object[])current, stack, tracker);
/*     */       }
/* 107 */       else if (((current instanceof ByteBuffer)) && (((ByteBuffer)current).hasArray()) && (!this.includeFullBufferSize))
/*     */       {
/*     */ 
/*     */ 
/* 111 */         ByteBuffer buffer = (ByteBuffer)current;
/*     */         
/* 113 */         total += 12 + buffer.remaining();
/*     */       } else {
/* 115 */         addFieldChildren(current, stack, tracker);
/*     */       }
/*     */     }
/*     */     
/* 119 */     return total;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public long countChildren(Object object)
/*     */   {
/* 127 */     if (object == null) {
/* 128 */       throw new NullPointerException();
/*     */     }
/*     */     
/* 131 */     Set<Object> tracker = java.util.Collections.newSetFromMap(new java.util.IdentityHashMap());
/* 132 */     tracker.add(object);
/* 133 */     Stack<Object> stack = new Stack();
/* 134 */     stack.push(object);
/*     */     
/* 136 */     long total = 0L;
/* 137 */     while (!stack.isEmpty()) {
/* 138 */       Object current = stack.pop();
/* 139 */       assert (current != null);
/* 140 */       total += 1L;
/*     */       
/* 142 */       if ((current instanceof Object[])) {
/* 143 */         addArrayChildren((Object[])current, stack, tracker);
/*     */       }
/*     */       else {
/* 146 */         addFieldChildren(current, stack, tracker);
/*     */       }
/*     */     }
/*     */     
/* 150 */     return total;
/*     */   }
/*     */   
/*     */   private void addFieldChildren(Object current, Stack<Object> stack, Set<Object> tracker) {
/* 154 */     Class cls = current.getClass();
/* 155 */     while (cls != null) {
/* 156 */       for (Field field : cls.getDeclaredFields()) {
/* 157 */         if ((!field.getType().isPrimitive()) && (!java.lang.reflect.Modifier.isStatic(field.getModifiers())))
/*     */         {
/*     */ 
/*     */ 
/* 161 */           field.setAccessible(true);
/*     */           Object child;
/*     */           try {
/* 164 */             child = field.get(current);
/*     */           }
/*     */           catch (IllegalAccessException e) {
/* 167 */             throw new RuntimeException(e);
/*     */           }
/*     */           
/* 170 */           if ((child != null) && (!tracker.contains(child))) {
/* 171 */             stack.push(child);
/* 172 */             tracker.add(child);
/*     */           }
/*     */         }
/*     */       }
/* 176 */       cls = cls.getSuperclass();
/*     */     }
/*     */   }
/*     */   
/*     */   private void addArrayChildren(Object[] current, Stack<Object> stack, Set<Object> tracker) {
/* 181 */     for (Object child : current) {
/* 182 */       if ((child != null) && (!tracker.contains(child))) {
/* 183 */         stack.push(child);
/* 184 */         tracker.add(child);
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\github\jamm\MemoryMeter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */