package prefuse.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import prefuse.data.column.Column;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.event.ExpressionListener;
import prefuse.data.event.ProjectionListener;
import prefuse.data.event.TableListener;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.Expression;
import prefuse.data.expression.Predicate;
import prefuse.data.tuple.TableTuple;
import prefuse.data.util.AcceptAllColumnProjection;
import prefuse.data.util.CascadedRowManager;
import prefuse.data.util.ColumnProjection;
import prefuse.data.util.RowManager;
import prefuse.util.collections.CompositeIterator;
import prefuse.util.collections.IntIterator;

public class CascadedTable
  extends Table
{
  protected Table m_parent;
  protected ArrayList m_pnames;
  protected ColumnProjection m_colFilter;
  protected Predicate m_rowFilter;
  protected Listener m_listener;
  
  public CascadedTable(Table paramTable)
  {
    this(paramTable, null, null);
  }
  
  public CascadedTable(Table paramTable, Predicate paramPredicate)
  {
    this(paramTable, paramPredicate, null);
  }
  
  public CascadedTable(Table paramTable, ColumnProjection paramColumnProjection)
  {
    this(paramTable, null, paramColumnProjection);
  }
  
  public CascadedTable(Table paramTable, Predicate paramPredicate, ColumnProjection paramColumnProjection)
  {
    this(paramTable, paramPredicate, paramColumnProjection, TableTuple.class);
  }
  
  protected CascadedTable(Table paramTable, Predicate paramPredicate, ColumnProjection paramColumnProjection, Class paramClass)
  {
    super(0, 0, paramClass);
    this.m_parent = paramTable;
    this.m_pnames = new ArrayList();
    this.m_rows = new CascadedRowManager(this);
    this.m_rowFilter = (paramPredicate == null ? BooleanLiteral.TRUE : paramPredicate);
    this.m_colFilter = (paramColumnProjection == null ? new AcceptAllColumnProjection() : paramColumnProjection);
    filterColumns();
    filterRows();
    this.m_listener = new Listener(null);
    this.m_parent.addTableListener(this.m_listener);
    if (this.m_rowFilter != BooleanLiteral.TRUE) {
      this.m_rowFilter.addExpressionListener(this.m_listener);
    }
    this.m_colFilter.addProjectionListener(this.m_listener);
  }
  
  protected CascadedTable()
  {
    this(TableTuple.class);
  }
  
  protected CascadedTable(Class paramClass)
  {
    super(0, 0, paramClass);
    this.m_pnames = new ArrayList();
  }
  
  protected void filterColumns()
  {
    if (this.m_parent == null) {
      return;
    }
    for (int i = 0; i < this.m_pnames.size(); i++)
    {
      String str1 = (String)this.m_pnames.get(i);
      Column localColumn1 = this.m_parent.getColumn(i);
      boolean bool = this.m_names.contains(str1);
      if ((!this.m_colFilter.include(localColumn1, str1)) || (bool))
      {
        this.m_pnames.remove(i--);
        if (!bool)
        {
          ((Table.ColumnEntry)this.m_entries.get(str1)).dispose();
          this.m_entries.remove(str1);
        }
        fireTableEvent(this.m_rows.getMinimumRow(), this.m_rows.getMaximumRow(), i, -1);
      }
    }
    this.m_pnames.clear();
    Iterator localIterator = this.m_parent.getColumnNames();
    int j = 0;
    int k = this.m_columns.size();
    while (localIterator.hasNext())
    {
      String str2 = (String)localIterator.next();
      Column localColumn2 = this.m_parent.getColumn(j);
      if ((this.m_colFilter.include(localColumn2, str2)) && (!this.m_names.contains(str2)))
      {
        this.m_pnames.add(str2);
        Table.ColumnEntry localColumnEntry = (Table.ColumnEntry)this.m_entries.get(str2);
        if (localColumnEntry == null)
        {
          localColumnEntry = new Table.ColumnEntry(k++, localColumn2, new ColumnMetadata(this, str2));
          this.m_entries.put(str2, localColumnEntry);
          fireTableEvent(this.m_rows.getMinimumRow(), this.m_rows.getMaximumRow(), j, 1);
        }
        else
        {
          localColumnEntry.colnum = (k++);
        }
        this.m_lastCol = (this.m_columns.size() - 1);
      }
      j++;
    }
  }
  
  public void filterRows()
  {
    if (this.m_parent == null) {
      return;
    }
    CascadedRowManager localCascadedRowManager = (CascadedRowManager)this.m_rows;
    IntIterator localIntIterator = this.m_rows.rows();
    while (localIntIterator.hasNext())
    {
      int i = localIntIterator.nextInt();
      if (!this.m_rowFilter.getBoolean(this.m_parent.getTuple(localCascadedRowManager.getParentRow(i)))) {
        removeCascadedRow(i);
      }
    }
    Iterator localIterator = this.m_parent.tuples(this.m_rowFilter);
    while (localIterator.hasNext())
    {
      Tuple localTuple = (Tuple)localIterator.next();
      int j = localTuple.getRow();
      if (localCascadedRowManager.getChildRow(j) == -1) {
        addCascadedRow(j);
      }
    }
  }
  
  public int getColumnCount()
  {
    return this.m_columns.size() + this.m_pnames.size();
  }
  
  public int getLocalColumnCount()
  {
    return this.m_columns.size();
  }
  
  public Table getParentTable()
  {
    return this.m_parent;
  }
  
  public int getParentRow(int paramInt)
  {
    return ((CascadedRowManager)this.m_rows).getParentRow(paramInt);
  }
  
  public int getChildRow(int paramInt)
  {
    return ((CascadedRowManager)this.m_rows).getChildRow(paramInt);
  }
  
  public int addRow()
  {
    if (this.m_parent != null) {
      throw new IllegalStateException("Add row not supported for CascadedTable.");
    }
    return super.addRow();
  }
  
  public void addRows(int paramInt)
  {
    if (this.m_parent != null) {
      throw new IllegalStateException("Add rows not supported for CascadedTable.");
    }
    super.addRows(paramInt);
  }
  
  public boolean removeRow(int paramInt)
  {
    if (this.m_parent != null) {
      throw new IllegalStateException("Remove row not supported for CascadedTable.");
    }
    return super.removeRow(paramInt);
  }
  
  protected int addCascadedRow(int paramInt)
  {
    int i = this.m_rows.addRow();
    ((CascadedRowManager)this.m_rows).put(i, paramInt);
    updateRowCount();
    fireTableEvent(i, i, -1, 1);
    return i;
  }
  
  protected boolean removeCascadedRow(int paramInt)
  {
    boolean bool = super.removeRow(paramInt);
    if (bool) {
      ((CascadedRowManager)this.m_rows).remove(paramInt);
    }
    return bool;
  }
  
  public String getColumnName(int paramInt)
  {
    int i = this.m_names.size();
    if (paramInt >= i) {
      return (String)this.m_pnames.get(paramInt - i);
    }
    return (String)this.m_names.get(paramInt);
  }
  
  public int getColumnNumber(Column paramColumn)
  {
    int i = this.m_columns.indexOf(paramColumn);
    if ((i == -1) && (this.m_parent != null))
    {
      i = this.m_parent.getColumnNumber(paramColumn);
      if (i == -1) {
        return i;
      }
      String str = this.m_parent.getColumnName(i);
      i = this.m_pnames.indexOf(str);
      if (i != -1) {
        i += this.m_columns.size();
      }
    }
    return i;
  }
  
  public Column getColumn(int paramInt)
  {
    this.m_lastCol = paramInt;
    int i = this.m_names.size();
    if ((paramInt >= i) && (this.m_parent != null)) {
      return this.m_parent.getColumn((String)this.m_pnames.get(paramInt - i));
    }
    return (Column)this.m_columns.get(paramInt);
  }
  
  protected boolean hasColumn(String paramString)
  {
    int i = getColumnNumber(paramString);
    return (i >= 0) && (i < getLocalColumnCount());
  }
  
  protected Iterator getColumnNames()
  {
    if (this.m_parent == null) {
      return this.m_names.iterator();
    }
    return new CompositeIterator(this.m_names.iterator(), this.m_pnames.iterator());
  }
  
  protected void invalidateSchema()
  {
    super.invalidateSchema();
    filterColumns();
  }
  
  private class Listener
    implements TableListener, ProjectionListener, ExpressionListener
  {
    private Listener() {}
    
    public void tableChanged(Table paramTable, int paramInt1, int paramInt2, int paramInt3, int paramInt4)
    {
      if (paramTable != CascadedTable.this.m_parent) {
        return;
      }
      CascadedRowManager localCascadedRowManager = (CascadedRowManager)CascadedTable.this.m_rows;
      int i;
      int j;
      switch (paramInt4)
      {
      case 0: 
        i = paramInt1;
        j = -1;
        while (i <= paramInt2)
        {
          if ((j = localCascadedRowManager.getChildRow(i)) != -1)
          {
            if (CascadedTable.this.m_rowFilter.getBoolean(CascadedTable.this.m_parent.getTuple(i)))
            {
              int k = CascadedTable.this.getColumnNumber(CascadedTable.this.m_parent.getColumnName(paramInt3));
              if (k >= CascadedTable.this.getLocalColumnCount()) {
                CascadedTable.this.fireTableEvent(j, j, k, 0);
              }
            }
            else
            {
              CascadedTable.this.removeCascadedRow(j);
            }
          }
          else if ((CascadedTable.this.m_rowFilter.getBoolean(CascadedTable.this.m_parent.getTuple(i))) && ((j = localCascadedRowManager.getChildRow(i)) < 0)) {
            CascadedTable.this.addCascadedRow(i);
          }
          i++;
        }
        break;
      case -1: 
        if (paramInt3 == -1)
        {
          i = paramInt1;
          j = -1;
          while (i <= paramInt2)
          {
            if ((j = localCascadedRowManager.getChildRow(i)) != -1) {
              CascadedTable.this.removeCascadedRow(j);
            }
            i++;
          }
        }
        else
        {
          CascadedTable.this.filterColumns();
        }
        break;
      case 1: 
        if (paramInt3 == -1) {
          for (i = paramInt1; i <= paramInt2; i++) {
            if ((CascadedTable.this.m_rowFilter.getBoolean(CascadedTable.this.m_parent.getTuple(i))) && (localCascadedRowManager.getChildRow(i) < 0)) {
              CascadedTable.this.addCascadedRow(i);
            }
          }
        } else {
          CascadedTable.this.filterColumns();
        }
        break;
      }
    }
    
    public void projectionChanged(ColumnProjection paramColumnProjection)
    {
      if (paramColumnProjection == CascadedTable.this.m_colFilter) {
        CascadedTable.this.filterColumns();
      }
    }
    
    public void expressionChanged(Expression paramExpression)
    {
      if (paramExpression == CascadedTable.this.m_rowFilter) {
        CascadedTable.this.filterRows();
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\CascadedTable.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */