package prefuse.data.expression;

import java.util.ArrayList;
import java.util.Iterator;

public abstract class CompositePredicate
  extends AbstractPredicate
{
  protected ArrayList m_clauses = new ArrayList(2);
  
  public CompositePredicate() {}
  
  public CompositePredicate(Predicate paramPredicate1, Predicate paramPredicate2)
  {
    this.m_clauses.add(paramPredicate1);
    this.m_clauses.add(paramPredicate2);
  }
  
  public void add(Predicate paramPredicate)
  {
    if (this.m_clauses.contains(paramPredicate)) {
      throw new IllegalArgumentException("Duplicate predicate.");
    }
    this.m_clauses.add(paramPredicate);
    fireExpressionChange();
  }
  
  public boolean remove(Predicate paramPredicate)
  {
    if (this.m_clauses.remove(paramPredicate))
    {
      fireExpressionChange();
      return true;
    }
    return false;
  }
  
  public void clear()
  {
    removeChildListeners();
    this.m_clauses.clear();
    fireExpressionChange();
  }
  
  public int size()
  {
    return this.m_clauses.size();
  }
  
  public Predicate get(int paramInt)
  {
    return (Predicate)this.m_clauses.get(paramInt);
  }
  
  public void set(Predicate paramPredicate)
  {
    removeChildListeners();
    this.m_clauses.clear();
    this.m_clauses.add(paramPredicate);
    if (hasListeners()) {
      addChildListeners();
    }
    fireExpressionChange();
  }
  
  public void set(Predicate[] paramArrayOfPredicate)
  {
    removeChildListeners();
    this.m_clauses.clear();
    for (int i = 0; i < paramArrayOfPredicate.length; i++) {
      if (!this.m_clauses.contains(paramArrayOfPredicate)) {
        this.m_clauses.add(paramArrayOfPredicate[i]);
      }
    }
    if (hasListeners()) {
      addChildListeners();
    }
    fireExpressionChange();
  }
  
  public Predicate getSubPredicate(Predicate paramPredicate)
  {
    CompositePredicate localCompositePredicate = null;
    try
    {
      localCompositePredicate = (CompositePredicate)getClass().newInstance();
    }
    catch (InstantiationException localInstantiationException) {}catch (IllegalAccessException localIllegalAccessException) {}
    for (int i = 0; i < this.m_clauses.size(); i++)
    {
      Predicate localPredicate = (Predicate)this.m_clauses.get(i);
      if (paramPredicate != localPredicate) {
        localCompositePredicate.add(localPredicate);
      }
    }
    return localCompositePredicate;
  }
  
  public void visit(ExpressionVisitor paramExpressionVisitor)
  {
    paramExpressionVisitor.visitExpression(this);
    Iterator localIterator = this.m_clauses.iterator();
    while (localIterator.hasNext())
    {
      paramExpressionVisitor.down();
      ((Expression)localIterator.next()).visit(paramExpressionVisitor);
      paramExpressionVisitor.up();
    }
  }
  
  protected void addChildListeners()
  {
    Iterator localIterator = this.m_clauses.iterator();
    while (localIterator.hasNext()) {
      ((Expression)localIterator.next()).addExpressionListener(this);
    }
  }
  
  protected void removeChildListeners()
  {
    Iterator localIterator = this.m_clauses.iterator();
    while (localIterator.hasNext()) {
      ((Expression)localIterator.next()).removeExpressionListener(this);
    }
  }
  
  protected String toString(String paramString)
  {
    if (this.m_clauses.size() == 1) {
      return this.m_clauses.get(0).toString();
    }
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append('(');
    Iterator localIterator = this.m_clauses.iterator();
    while (localIterator.hasNext())
    {
      localStringBuffer.append(localIterator.next().toString());
      if (localIterator.hasNext())
      {
        localStringBuffer.append(" ");
        localStringBuffer.append(paramString);
        localStringBuffer.append(" ");
      }
    }
    localStringBuffer.append(')');
    return localStringBuffer.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\CompositePredicate.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */