package prefuse.render;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.CubicCurve2D.Float;
import java.awt.geom.Line2D;
import java.awt.geom.Line2D.Float;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Float;
import java.awt.geom.Rectangle2D;
import prefuse.util.ColorLib;
import prefuse.util.GraphicsLib;
import prefuse.util.StrokeLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class EdgeRenderer
  extends AbstractShapeRenderer
{
  public static final String EDGE_TYPE = "edgeType";
  protected static final double HALF_PI = 1.5707963267948966D;
  protected Line2D m_line = new Line2D.Float();
  protected CubicCurve2D m_cubic = new CubicCurve2D.Float();
  protected int m_edgeType = 0;
  protected int m_xAlign1 = 2;
  protected int m_yAlign1 = 2;
  protected int m_xAlign2 = 2;
  protected int m_yAlign2 = 2;
  protected double m_width = 1.0D;
  protected float m_curWidth = 1.0F;
  protected Point2D[] m_tmpPoints = new Point2D[2];
  protected Point2D[] m_ctrlPoints = new Point2D[2];
  protected Point2D[] m_isctPoints = new Point2D[2];
  protected int m_edgeArrow = 1;
  protected int m_arrowWidth = 8;
  protected int m_arrowHeight = 12;
  protected Polygon m_arrowHead = updateArrowHead(this.m_arrowWidth, this.m_arrowHeight);
  protected AffineTransform m_arrowTrans = new AffineTransform();
  protected Shape m_curArrow;
  
  public EdgeRenderer()
  {
    this.m_tmpPoints[0] = new Point2D.Float();
    this.m_tmpPoints[1] = new Point2D.Float();
    this.m_ctrlPoints[0] = new Point2D.Float();
    this.m_ctrlPoints[1] = new Point2D.Float();
    this.m_isctPoints[0] = new Point2D.Float();
    this.m_isctPoints[1] = new Point2D.Float();
  }
  
  public EdgeRenderer(int paramInt)
  {
    this(paramInt, 1);
  }
  
  public EdgeRenderer(int paramInt1, int paramInt2)
  {
    this();
    setEdgeType(paramInt1);
    setArrowType(paramInt2);
  }
  
  public int getRenderType(VisualItem paramVisualItem)
  {
    return 1;
  }
  
  protected Shape getRawShape(VisualItem paramVisualItem)
  {
    EdgeItem localEdgeItem1 = (EdgeItem)paramVisualItem;
    NodeItem localNodeItem1 = localEdgeItem1.getSourceItem();
    NodeItem localNodeItem2 = localEdgeItem1.getTargetItem();
    int i = this.m_edgeType;
    getAlignedPoint(this.m_tmpPoints[0], localNodeItem1.getBounds(), this.m_xAlign1, this.m_yAlign1);
    getAlignedPoint(this.m_tmpPoints[1], localNodeItem2.getBounds(), this.m_xAlign2, this.m_yAlign2);
    this.m_curWidth = ((float)(this.m_width * getLineWidth(paramVisualItem)));
    EdgeItem localEdgeItem2 = (EdgeItem)paramVisualItem;
    if ((localEdgeItem2.isDirected()) && (this.m_edgeArrow != 0))
    {
      int j = this.m_edgeArrow == 1 ? 1 : 0;
      Point2D localPoint2D1 = null;
      Point2D localPoint2D2 = null;
      localPoint2D1 = this.m_tmpPoints[1];
      localPoint2D2 = this.m_tmpPoints[0];
      NodeItem localNodeItem3 = j != 0 ? localEdgeItem2.getTargetItem() : localEdgeItem2.getSourceItem();
      int k = GraphicsLib.intersectLineRectangle(localPoint2D1, localPoint2D2, localNodeItem3.getBounds(), this.m_isctPoints);
      if (k > 0) {
        localPoint2D2 = this.m_isctPoints[0];
      }
      AffineTransform localAffineTransform = getArrowTrans(localPoint2D1, localPoint2D2, this.m_curWidth);
      this.m_curArrow = localAffineTransform.createTransformedShape(this.m_arrowHead);
      Point2D localPoint2D3 = this.m_tmpPoints[0];
      localPoint2D3.setLocation(0.0D, -this.m_arrowHeight);
      localAffineTransform.transform(localPoint2D3, localPoint2D3);
    }
    else
    {
      this.m_curArrow = null;
    }
    Object localObject = null;
    double d1 = this.m_tmpPoints[0].getX();
    double d2 = this.m_tmpPoints[0].getY();
    double d3 = this.m_tmpPoints[1].getX();
    double d4 = this.m_tmpPoints[1].getY();
    switch (i)
    {
    case 0: 
      this.m_line.setLine(d1, d2, d3, d4);
      localObject = this.m_line;
      break;
    case 1: 
      getCurveControlPoints(localEdgeItem1, this.m_ctrlPoints, d1, d2, d3, d4);
      this.m_cubic.setCurve(d1, d2, this.m_ctrlPoints[0].getX(), this.m_ctrlPoints[0].getY(), this.m_ctrlPoints[1].getX(), this.m_ctrlPoints[1].getY(), d3, d4);
      localObject = this.m_cubic;
      break;
    default: 
      throw new IllegalStateException("Unknown edge type");
    }
    return (Shape)localObject;
  }
  
  public void render(Graphics2D paramGraphics2D, VisualItem paramVisualItem)
  {
    super.render(paramGraphics2D, paramVisualItem);
    if (this.m_curArrow != null)
    {
      paramGraphics2D.setPaint(ColorLib.getColor(paramVisualItem.getFillColor()));
      paramGraphics2D.fill(this.m_curArrow);
    }
  }
  
  protected AffineTransform getArrowTrans(Point2D paramPoint2D1, Point2D paramPoint2D2, double paramDouble)
  {
    this.m_arrowTrans.setToTranslation(paramPoint2D2.getX(), paramPoint2D2.getY());
    this.m_arrowTrans.rotate(-1.5707963267948966D + Math.atan2(paramPoint2D2.getY() - paramPoint2D1.getY(), paramPoint2D2.getX() - paramPoint2D1.getX()));
    if (paramDouble > 1.0D)
    {
      double d = paramDouble / 4.0D;
      this.m_arrowTrans.scale(d, d);
    }
    return this.m_arrowTrans;
  }
  
  protected Polygon updateArrowHead(int paramInt1, int paramInt2)
  {
    if (this.m_arrowHead == null) {
      this.m_arrowHead = new Polygon();
    } else {
      this.m_arrowHead.reset();
    }
    this.m_arrowHead.addPoint(0, 0);
    this.m_arrowHead.addPoint(-paramInt1 / 2, -paramInt2);
    this.m_arrowHead.addPoint(paramInt1 / 2, -paramInt2);
    this.m_arrowHead.addPoint(0, 0);
    return this.m_arrowHead;
  }
  
  protected AffineTransform getTransform(VisualItem paramVisualItem)
  {
    return null;
  }
  
  public boolean locatePoint(Point2D paramPoint2D, VisualItem paramVisualItem)
  {
    Shape localShape = getShape(paramVisualItem);
    if (localShape == null) {
      return false;
    }
    double d1 = Math.max(2.0D, getLineWidth(paramVisualItem));
    double d2 = d1 / 2.0D;
    return localShape.intersects(paramPoint2D.getX() - d2, paramPoint2D.getY() - d2, d1, d1);
  }
  
  public void setBounds(VisualItem paramVisualItem)
  {
    if (!this.m_manageBounds) {
      return;
    }
    Shape localShape = getShape(paramVisualItem);
    if (localShape == null)
    {
      paramVisualItem.setBounds(paramVisualItem.getX(), paramVisualItem.getY(), 0.0D, 0.0D);
      return;
    }
    GraphicsLib.setBounds(paramVisualItem, localShape, getStroke(paramVisualItem));
    if (this.m_curArrow != null)
    {
      Rectangle2D localRectangle2D = (Rectangle2D)paramVisualItem.get(VisualItem.BOUNDS);
      Rectangle2D.union(localRectangle2D, this.m_curArrow.getBounds2D(), localRectangle2D);
    }
  }
  
  protected double getLineWidth(VisualItem paramVisualItem)
  {
    return paramVisualItem.getSize();
  }
  
  protected BasicStroke getStroke(VisualItem paramVisualItem)
  {
    return StrokeLib.getDerivedStroke(paramVisualItem.getStroke(), this.m_curWidth);
  }
  
  protected void getCurveControlPoints(EdgeItem paramEdgeItem, Point2D[] paramArrayOfPoint2D, double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    double d1 = paramDouble3 - paramDouble1;
    double d2 = paramDouble4 - paramDouble2;
    paramArrayOfPoint2D[0].setLocation(paramDouble1 + 2.0D * d1 / 3.0D, paramDouble2);
    paramArrayOfPoint2D[1].setLocation(paramDouble3 - d1 / 8.0D, paramDouble4 - d2 / 8.0D);
  }
  
  protected static void getAlignedPoint(Point2D paramPoint2D, Rectangle2D paramRectangle2D, int paramInt1, int paramInt2)
  {
    double d1 = paramRectangle2D.getX();
    double d2 = paramRectangle2D.getY();
    double d3 = paramRectangle2D.getWidth();
    double d4 = paramRectangle2D.getHeight();
    if (paramInt1 == 2) {
      d1 += d3 / 2.0D;
    } else if (paramInt1 == 1) {
      d1 += d3;
    }
    if (paramInt2 == 2) {
      d2 += d4 / 2.0D;
    } else if (paramInt2 == 1) {
      d2 += d4;
    }
    paramPoint2D.setLocation(d1, d2);
  }
  
  public int getEdgeType()
  {
    return this.m_edgeType;
  }
  
  public void setEdgeType(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 2)) {
      throw new IllegalArgumentException("Unrecognized edge curve type: " + paramInt);
    }
    this.m_edgeType = paramInt;
  }
  
  public int getArrowType()
  {
    return this.m_edgeArrow;
  }
  
  public void setArrowType(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 3)) {
      throw new IllegalArgumentException("Unrecognized edge arrow type: " + paramInt);
    }
    this.m_edgeArrow = paramInt;
  }
  
  public void setArrowHeadSize(int paramInt1, int paramInt2)
  {
    this.m_arrowWidth = paramInt1;
    this.m_arrowHeight = paramInt2;
    this.m_arrowHead = updateArrowHead(paramInt1, paramInt2);
  }
  
  public int getArrowHeadHeight()
  {
    return this.m_arrowHeight;
  }
  
  public int getArrowHeadWidth()
  {
    return this.m_arrowWidth;
  }
  
  public int getHorizontalAlignment1()
  {
    return this.m_xAlign1;
  }
  
  public int getVerticalAlignment1()
  {
    return this.m_yAlign1;
  }
  
  public int getHorizontalAlignment2()
  {
    return this.m_xAlign2;
  }
  
  public int getVerticalAlignment2()
  {
    return this.m_yAlign2;
  }
  
  public void setHorizontalAlignment1(int paramInt)
  {
    this.m_xAlign1 = paramInt;
  }
  
  public void setVerticalAlignment1(int paramInt)
  {
    this.m_yAlign1 = paramInt;
  }
  
  public void setHorizontalAlignment2(int paramInt)
  {
    this.m_xAlign2 = paramInt;
  }
  
  public void setVerticalAlignment2(int paramInt)
  {
    this.m_yAlign2 = paramInt;
  }
  
  public void setDefaultLineWidth(double paramDouble)
  {
    this.m_width = paramDouble;
  }
  
  public double getDefaultLineWidth()
  {
    return this.m_width;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\EdgeRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */