package prefuse.util;

import java.util.ArrayList;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tree;

public class GraphLib
{
  public static final String LABEL = "label";
  public static final Schema LABEL_SCHEMA = new Schema();
  
  public static Graph getNodes(int paramInt)
  {
    Graph localGraph = new Graph();
    localGraph.getNodeTable().addColumns(LABEL_SCHEMA);
    for (int i = 0; i < paramInt; i++)
    {
      Node localNode = localGraph.addNode();
      localNode.setString("label", String.valueOf(i));
    }
    return localGraph;
  }
  
  public static Graph getStar(int paramInt)
  {
    Graph localGraph = new Graph();
    localGraph.getNodeTable().addColumns(LABEL_SCHEMA);
    Node localNode1 = localGraph.addNode();
    localNode1.setString("label", "0");
    for (int i = 1; i <= paramInt; i++)
    {
      Node localNode2 = localGraph.addNode();
      localNode2.setString("label", String.valueOf(i));
      localGraph.addEdge(localNode1, localNode2);
    }
    return localGraph;
  }
  
  public static Graph getClique(int paramInt)
  {
    Graph localGraph = new Graph();
    localGraph.getNodeTable().addColumns(LABEL_SCHEMA);
    Node[] arrayOfNode = new Node[paramInt];
    for (int i = 0; i < paramInt; i++)
    {
      arrayOfNode[i] = localGraph.addNode();
      arrayOfNode[i].setString("label", String.valueOf(i));
    }
    for (i = 0; i < paramInt; i++) {
      for (int j = i; j < paramInt; j++) {
        if (i != j) {
          localGraph.addEdge(arrayOfNode[i], arrayOfNode[j]);
        }
      }
    }
    return localGraph;
  }
  
  public static Graph getGrid(int paramInt1, int paramInt2)
  {
    Graph localGraph = new Graph();
    localGraph.getNodeTable().addColumns(LABEL_SCHEMA);
    Node[] arrayOfNode = new Node[paramInt1 * paramInt2];
    for (int i = 0; i < paramInt1 * paramInt2; i++)
    {
      arrayOfNode[i] = localGraph.addNode();
      arrayOfNode[i].setString("label", String.valueOf(i));
      if (i >= paramInt2) {
        localGraph.addEdge(arrayOfNode[(i - paramInt2)], arrayOfNode[i]);
      }
      if (i % paramInt2 != 0) {
        localGraph.addEdge(arrayOfNode[(i - 1)], arrayOfNode[i]);
      }
    }
    return localGraph;
  }
  
  public static Graph getHoneycomb(int paramInt)
  {
    Graph localGraph = new Graph();
    localGraph.getNodeTable().addColumns(LABEL_SCHEMA);
    ArrayList localArrayList1 = halfcomb(localGraph, paramInt);
    ArrayList localArrayList2 = halfcomb(localGraph, paramInt);
    for (int i = 0; i < paramInt << 1; i++)
    {
      Node localNode1 = (Node)localArrayList1.get(i);
      Node localNode2 = (Node)localArrayList2.get(i);
      localGraph.addEdge(localNode1, localNode2);
    }
    return localGraph;
  }
  
  private static ArrayList halfcomb(Graph paramGraph, int paramInt)
  {
    ArrayList localArrayList1 = new ArrayList();
    ArrayList localArrayList2 = new ArrayList();
    int i = 0;
    Node localNode1;
    for (int j = 0; j < paramInt; j++)
    {
      localNode1 = paramGraph.addNode();
      localNode1.setString("label", String.valueOf(i++));
      localArrayList1.add(localNode1);
    }
    for (j = 0; j < paramInt; j++)
    {
      localNode1 = null;
      Node localNode2;
      for (int k = 0; k < localArrayList1.size(); k++)
      {
        localNode2 = (Node)localArrayList1.get(k);
        if (localNode1 == null)
        {
          localNode1 = paramGraph.addNode();
          localNode1.setString("label", String.valueOf(i++));
          localArrayList2.add(localNode1);
        }
        paramGraph.addEdge(localNode2, localNode1);
        localNode1 = paramGraph.addNode();
        localNode1.setString("label", String.valueOf(i++));
        localArrayList2.add(localNode1);
        paramGraph.addEdge(localNode2, localNode1);
      }
      if (j == paramInt - 1) {
        return localArrayList2;
      }
      localArrayList1.clear();
      for (k = 0; k < localArrayList2.size(); k++)
      {
        localNode2 = (Node)localArrayList2.get(k);
        localNode1 = paramGraph.addNode();
        localNode1.setString("label", String.valueOf(i++));
        localArrayList1.add(localNode1);
        paramGraph.addEdge(localNode2, localNode1);
      }
      localArrayList2.clear();
    }
    return localArrayList1;
  }
  
  public static Tree getBalancedTree(int paramInt1, int paramInt2)
  {
    Tree localTree = new Tree();
    localTree.getNodeTable().addColumns(LABEL_SCHEMA);
    Node localNode = localTree.addRoot();
    localNode.setString("label", "0,0");
    if (paramInt2 > 0) {
      balancedHelper(localTree, localNode, paramInt1, paramInt2 - 1);
    }
    return localTree;
  }
  
  private static void balancedHelper(Tree paramTree, Node paramNode, int paramInt1, int paramInt2)
  {
    for (int i = 0; i < paramInt1; i++)
    {
      Node localNode = paramTree.addChild(paramNode);
      localNode.setString("label", i + "," + localNode.getDepth());
      if (paramInt2 > 0) {
        balancedHelper(paramTree, localNode, paramInt1, paramInt2 - 1);
      }
    }
  }
  
  public static Tree getLeftDeepTree(int paramInt)
  {
    Tree localTree = new Tree();
    localTree.getNodeTable().addColumns(LABEL_SCHEMA);
    Node localNode = localTree.addRoot();
    localNode.setString("label", "0,0");
    deepHelper(localTree, localNode, 2, paramInt, true);
    return localTree;
  }
  
  public static Tree getRightDeepTree(int paramInt)
  {
    Tree localTree = new Tree();
    localTree.getNodeTable().addColumns(LABEL_SCHEMA);
    Node localNode = localTree.addRoot();
    localNode.setString("label", "0,0");
    deepHelper(localTree, localNode, 2, paramInt, false);
    return localTree;
  }
  
  public static Tree getDiamondTree(int paramInt1, int paramInt2, int paramInt3)
  {
    Tree localTree = new Tree();
    localTree.getNodeTable().addColumns(LABEL_SCHEMA);
    Node localNode1 = localTree.addRoot();
    localNode1.setString("label", "0,0");
    Node localNode2 = localTree.addChild(localNode1);
    localNode2.setString("label", "1,0");
    Node localNode3 = localTree.addChild(localNode1);
    localNode3.setString("label", "1,1");
    deepHelper(localTree, localNode2, paramInt1, paramInt2 - 2, true);
    deepHelper(localTree, localNode3, paramInt1, paramInt2 - 2, false);
    while (localNode2.getFirstChild() != null) {
      localNode2 = localNode2.getFirstChild();
    }
    while (localNode3.getLastChild() != null) {
      localNode3 = localNode3.getLastChild();
    }
    deepHelper(localTree, localNode2, paramInt1, paramInt3 - 1, false);
    deepHelper(localTree, localNode3, paramInt1, paramInt3 - 1, true);
    return localTree;
  }
  
  private static void deepHelper(Tree paramTree, Node paramNode, int paramInt1, int paramInt2, boolean paramBoolean)
  {
    Node localNode = paramTree.addChild(paramNode);
    localNode.setString("label", "0," + localNode.getDepth());
    if ((paramBoolean) && (paramInt2 > 0)) {
      deepHelper(paramTree, localNode, paramInt1, paramInt2 - 1, paramBoolean);
    }
    for (int i = 1; i < paramInt1; i++)
    {
      localNode = paramTree.addChild(paramNode);
      localNode.setString("label", i + "," + localNode.getDepth());
    }
    if ((!paramBoolean) && (paramInt2 > 0)) {
      deepHelper(paramTree, localNode, paramInt1, paramInt2 - 1, paramBoolean);
    }
  }
  
  static
  {
    LABEL_SCHEMA.addColumn("label", String.class, "");
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\GraphLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */