package prefuse.util.collections;

import java.util.ArrayList;
import java.util.Arrays;

public class IntObjectHashMap
  extends AbstractHashMap
  implements Cloneable
{
  protected static final int defaultCapacity = 277;
  protected static final double defaultMinLoadFactor = 0.2D;
  protected static final double defaultMaxLoadFactor = 0.5D;
  protected static final byte FREE = 0;
  protected static final byte FULL = 1;
  protected static final byte REMOVED = 2;
  protected int[] table;
  protected Object[] values;
  protected byte[] state;
  protected int freeEntries;
  
  public IntObjectHashMap()
  {
    this(277);
  }
  
  public IntObjectHashMap(int paramInt)
  {
    this(paramInt, 0.2D, 0.5D);
  }
  
  public IntObjectHashMap(int paramInt, double paramDouble1, double paramDouble2)
  {
    setUp(paramInt, paramDouble1, paramDouble2);
  }
  
  public void clear()
  {
    Arrays.fill(this.state, (byte)0);
    Arrays.fill(this.values, null);
    this.distinct = 0;
    this.freeEntries = this.table.length;
    trimToSize();
  }
  
  public Object clone()
  {
    try
    {
      IntObjectHashMap localIntObjectHashMap = (IntObjectHashMap)super.clone();
      localIntObjectHashMap.table = ((int[])localIntObjectHashMap.table.clone());
      localIntObjectHashMap.values = ((Object[])localIntObjectHashMap.values.clone());
      localIntObjectHashMap.state = ((byte[])localIntObjectHashMap.state.clone());
      return localIntObjectHashMap;
    }
    catch (CloneNotSupportedException localCloneNotSupportedException) {}
    return null;
  }
  
  public boolean containsKey(int paramInt)
  {
    return indexOfKey(paramInt) >= 0;
  }
  
  public boolean containsValue(Object paramObject)
  {
    return indexOfValue(paramObject) >= 0;
  }
  
  public void ensureCapacity(int paramInt)
  {
    if (this.table.length < paramInt)
    {
      int i = nextPrime(paramInt);
      rehash(i);
    }
  }
  
  public Object get(int paramInt)
  {
    int i = indexOfKey(paramInt);
    if (i < 0) {
      return null;
    }
    return this.values[i];
  }
  
  protected int indexOfInsertion(int paramInt)
  {
    int[] arrayOfInt = this.table;
    byte[] arrayOfByte = this.state;
    int i = arrayOfInt.length;
    int j = paramInt & 0x7FFFFFFF;
    int k = j % i;
    int m = j % (i - 2);
    if (m == 0) {
      m = 1;
    }
    while ((arrayOfByte[k] == 1) && (arrayOfInt[k] != paramInt))
    {
      k -= m;
      if (k < 0) {
        k += i;
      }
    }
    if (arrayOfByte[k] == 2)
    {
      int n = k;
      while ((arrayOfByte[k] != 0) && ((arrayOfByte[k] == 2) || (arrayOfInt[k] != paramInt)))
      {
        k -= m;
        if (k < 0) {
          k += i;
        }
      }
      if (arrayOfByte[k] == 0) {
        k = n;
      }
    }
    if (arrayOfByte[k] == 1) {
      return -k - 1;
    }
    return k;
  }
  
  protected int indexOfKey(int paramInt)
  {
    int[] arrayOfInt = this.table;
    byte[] arrayOfByte = this.state;
    int i = arrayOfInt.length;
    int j = paramInt & 0x7FFFFFFF;
    int k = j % i;
    int m = j % (i - 2);
    if (m == 0) {
      m = 1;
    }
    while ((arrayOfByte[k] != 0) && ((arrayOfByte[k] == 2) || (arrayOfInt[k] != paramInt)))
    {
      k -= m;
      if (k < 0) {
        k += i;
      }
    }
    if (arrayOfByte[k] == 0) {
      return -1;
    }
    return k;
  }
  
  protected int indexOfValue(Object paramObject)
  {
    Object[] arrayOfObject = this.values;
    byte[] arrayOfByte = this.state;
    int i = arrayOfByte.length;
    do
    {
      i--;
      if (i < 0) {
        break;
      }
    } while ((arrayOfByte[i] != 1) || (arrayOfObject[i] != paramObject));
    return i;
    return -1;
  }
  
  public int keyOf(Object paramObject)
  {
    int i = indexOfValue(paramObject);
    if (i < 0) {
      return Integer.MIN_VALUE;
    }
    return this.table[i];
  }
  
  public int keys(int[] paramArrayOfInt)
  {
    int[] arrayOfInt = this.table;
    byte[] arrayOfByte = this.state;
    if (paramArrayOfInt.length < this.distinct) {
      return -1;
    }
    int i = 0;
    int j = arrayOfInt.length;
    while (j-- > 0) {
      if (arrayOfByte[j] == 1) {
        paramArrayOfInt[(i++)] = arrayOfInt[j];
      }
    }
    return this.distinct;
  }
  
  public boolean put(int paramInt, Object paramObject)
  {
    int i = indexOfInsertion(paramInt);
    if (i < 0)
    {
      i = -i - 1;
      this.values[i] = paramObject;
      return false;
    }
    int j;
    if (this.distinct > this.highWaterMark)
    {
      j = chooseGrowCapacity(this.distinct + 1, this.minLoadFactor, this.maxLoadFactor);
      rehash(j);
      return put(paramInt, paramObject);
    }
    this.table[i] = paramInt;
    this.values[i] = paramObject;
    if (this.state[i] == 0) {
      this.freeEntries -= 1;
    }
    this.state[i] = 1;
    this.distinct += 1;
    if (this.freeEntries < 1)
    {
      j = chooseGrowCapacity(this.distinct + 1, this.minLoadFactor, this.maxLoadFactor);
      rehash(j);
    }
    return true;
  }
  
  protected void rehash(int paramInt)
  {
    int i = this.table.length;
    int[] arrayOfInt1 = this.table;
    Object[] arrayOfObject1 = this.values;
    byte[] arrayOfByte1 = this.state;
    int[] arrayOfInt2 = new int[paramInt];
    Object[] arrayOfObject2 = new Object[paramInt];
    byte[] arrayOfByte2 = new byte[paramInt];
    this.lowWaterMark = chooseLowWaterMark(paramInt, this.minLoadFactor);
    this.highWaterMark = chooseHighWaterMark(paramInt, this.maxLoadFactor);
    this.table = arrayOfInt2;
    this.values = arrayOfObject2;
    this.state = arrayOfByte2;
    this.freeEntries = (paramInt - this.distinct);
    int j = i;
    while (j-- > 0) {
      if (arrayOfByte1[j] == 1)
      {
        int k = arrayOfInt1[j];
        int m = indexOfInsertion(k);
        arrayOfInt2[m] = k;
        arrayOfObject2[m] = arrayOfObject1[j];
        arrayOfByte2[m] = 1;
      }
    }
  }
  
  public boolean removeKey(int paramInt)
  {
    int i = indexOfKey(paramInt);
    if (i < 0) {
      return false;
    }
    this.state[i] = 2;
    this.values[i] = null;
    this.distinct -= 1;
    if (this.distinct < this.lowWaterMark)
    {
      int j = chooseShrinkCapacity(this.distinct, this.minLoadFactor, this.maxLoadFactor);
      rehash(j);
    }
    return true;
  }
  
  protected void setUp(int paramInt, double paramDouble1, double paramDouble2)
  {
    int i = paramInt;
    super.setUp(i, paramDouble1, paramDouble2);
    i = nextPrime(i);
    if (i == 0) {
      i = 1;
    }
    this.table = new int[i];
    this.values = new Object[i];
    this.state = new byte[i];
    this.minLoadFactor = paramDouble1;
    if (i == Integer.MAX_VALUE) {
      this.maxLoadFactor = 1.0D;
    } else {
      this.maxLoadFactor = paramDouble2;
    }
    this.distinct = 0;
    this.freeEntries = i;
    this.lowWaterMark = 0;
    this.highWaterMark = chooseHighWaterMark(i, this.maxLoadFactor);
  }
  
  public void trimToSize()
  {
    int i = nextPrime((int)(1.0D + 1.2D * size()));
    if (this.table.length > i) {
      rehash(i);
    }
  }
  
  public void values(ArrayList paramArrayList)
  {
    Object[] arrayOfObject = this.values;
    byte[] arrayOfByte = this.state;
    int i = arrayOfByte.length;
    while (i-- > 0) {
      if (arrayOfByte[i] == 1) {
        paramArrayList.add(arrayOfObject[i]);
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\collections\IntObjectHashMap.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */