package prefuse.util.io;

import java.io.PrintWriter;
import java.util.ArrayList;

public class XMLWriter
{
  private PrintWriter m_out;
  private int m_bias = 0;
  private int m_tab;
  private ArrayList m_tagStack = new ArrayList();
  private static final char LOWER_RANGE = ' ';
  private static final char UPPER_RANGE = '';
  private static final char[] VALID_CHARS = { '\t', '\n', '\r' };
  private static final char[] INVALID = { '<', '>', '"', '\'', '&' };
  private static final String[] VALID = { "&lt;", "&gt;", "&quot;", "&apos;", "&amp;" };
  
  public XMLWriter(PrintWriter paramPrintWriter)
  {
    this(paramPrintWriter, 2);
  }
  
  public XMLWriter(PrintWriter paramPrintWriter, int paramInt)
  {
    this.m_out = paramPrintWriter;
    this.m_tab = 2;
  }
  
  public void print(String paramString)
  {
    this.m_out.print(paramString);
  }
  
  public void println(String paramString)
  {
    this.m_out.print(paramString);
    this.m_out.print("\n");
  }
  
  public void println()
  {
    this.m_out.print("\n");
  }
  
  public void begin()
  {
    this.m_out.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    println();
  }
  
  public void begin(String paramString, int paramInt)
  {
    begin();
    this.m_out.print(paramString);
    this.m_bias = paramInt;
  }
  
  public void comment(String paramString)
  {
    spacing();
    this.m_out.print("<!-- ");
    this.m_out.print(paramString);
    this.m_out.print(" -->");
    println();
  }
  
  protected void tag(String paramString, String[] paramArrayOfString1, String[] paramArrayOfString2, int paramInt, boolean paramBoolean)
  {
    spacing();
    this.m_out.print('<');
    this.m_out.print(paramString);
    for (int i = 0; i < paramInt; i++)
    {
      this.m_out.print(' ');
      this.m_out.print(paramArrayOfString1[i]);
      this.m_out.print('=');
      this.m_out.print('"');
      escapeString(paramArrayOfString2[i]);
      this.m_out.print('"');
    }
    if (paramBoolean) {
      this.m_out.print('/');
    }
    this.m_out.print('>');
    println();
    if (!paramBoolean) {
      this.m_tagStack.add(paramString);
    }
  }
  
  public void tag(String paramString, String[] paramArrayOfString1, String[] paramArrayOfString2, int paramInt)
  {
    tag(paramString, paramArrayOfString1, paramArrayOfString2, paramInt, true);
  }
  
  public void start(String paramString, String[] paramArrayOfString1, String[] paramArrayOfString2, int paramInt)
  {
    tag(paramString, paramArrayOfString1, paramArrayOfString2, paramInt, false);
  }
  
  protected void tag(String paramString1, String paramString2, String paramString3, boolean paramBoolean)
  {
    spacing();
    this.m_out.print('<');
    this.m_out.print(paramString1);
    this.m_out.print(' ');
    this.m_out.print(paramString2);
    this.m_out.print('=');
    this.m_out.print('"');
    escapeString(paramString3);
    this.m_out.print('"');
    if (paramBoolean) {
      this.m_out.print('/');
    }
    this.m_out.print('>');
    println();
    if (!paramBoolean) {
      this.m_tagStack.add(paramString1);
    }
  }
  
  public void tag(String paramString1, String paramString2, String paramString3)
  {
    tag(paramString1, paramString2, paramString3, true);
  }
  
  public void start(String paramString1, String paramString2, String paramString3)
  {
    tag(paramString1, paramString2, paramString3, false);
  }
  
  protected void tag(String paramString, ArrayList paramArrayList1, ArrayList paramArrayList2, int paramInt, boolean paramBoolean)
  {
    spacing();
    this.m_out.print('<');
    this.m_out.print(paramString);
    for (int i = 0; i < paramInt; i++)
    {
      this.m_out.print(' ');
      this.m_out.print((String)paramArrayList1.get(i));
      this.m_out.print('=');
      this.m_out.print('"');
      escapeString((String)paramArrayList2.get(i));
      this.m_out.print('"');
    }
    if (paramBoolean) {
      this.m_out.print('/');
    }
    this.m_out.print('>');
    println();
    if (!paramBoolean) {
      this.m_tagStack.add(paramString);
    }
  }
  
  public void tag(String paramString, ArrayList paramArrayList1, ArrayList paramArrayList2, int paramInt)
  {
    tag(paramString, paramArrayList1, paramArrayList2, paramInt, true);
  }
  
  public void start(String paramString, ArrayList paramArrayList1, ArrayList paramArrayList2, int paramInt)
  {
    tag(paramString, paramArrayList1, paramArrayList2, paramInt, false);
  }
  
  public void start(String paramString)
  {
    tag(paramString, (String[])null, null, 0, false);
  }
  
  public void end()
  {
    String str = (String)this.m_tagStack.remove(this.m_tagStack.size() - 1);
    spacing();
    this.m_out.print('<');
    this.m_out.print('/');
    this.m_out.print(str);
    this.m_out.print('>');
    println();
  }
  
  public void contentTag(String paramString1, String paramString2, String paramString3, String paramString4)
  {
    spacing();
    this.m_out.print('<');
    this.m_out.print(paramString1);
    this.m_out.print(' ');
    this.m_out.print(paramString2);
    this.m_out.print('=');
    this.m_out.print('"');
    escapeString(paramString3);
    this.m_out.print('"');
    this.m_out.print('>');
    escapeString(paramString4);
    this.m_out.print('<');
    this.m_out.print('/');
    this.m_out.print(paramString1);
    this.m_out.print('>');
    println();
  }
  
  public void contentTag(String paramString1, String paramString2)
  {
    spacing();
    this.m_out.print('<');
    this.m_out.print(paramString1);
    this.m_out.print('>');
    escapeString(paramString2);
    this.m_out.print('<');
    this.m_out.print('/');
    this.m_out.print(paramString1);
    this.m_out.print('>');
    println();
  }
  
  public void content(String paramString)
  {
    escapeString(paramString);
  }
  
  public void finish()
  {
    this.m_bias = 0;
    this.m_out.flush();
  }
  
  public void finish(String paramString)
  {
    this.m_bias = 0;
    this.m_out.print(paramString);
    this.m_out.flush();
  }
  
  public void spacing()
  {
    int i = this.m_bias + this.m_tagStack.size() * this.m_tab;
    for (int j = 0; j < i; j++) {
      this.m_out.print(' ');
    }
  }
  
  protected void escapeString(String paramString)
  {
    if (paramString == null)
    {
      this.m_out.print("null");
      return;
    }
    int i = paramString.length();
    for (int j = 0; j < i; j++)
    {
      char c = paramString.charAt(j);
      if (((c < ' ') && (c != VALID_CHARS[0]) && (c != VALID_CHARS[1]) && (c != VALID_CHARS[2])) || (c > ''))
      {
        this.m_out.print("&#");
        this.m_out.print(Integer.toString(c));
        this.m_out.print(';');
      }
      else
      {
        int k = 1;
        for (int m = INVALID.length - 1; m >= 0; m--) {
          if (INVALID[m] == c)
          {
            k = 0;
            this.m_out.print(VALID[m]);
            break;
          }
        }
        if (k != 0) {
          this.m_out.print(c);
        }
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\io\XMLWriter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */